################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
sequences2graph.py - align sequences and put them into a graph
==============================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python sequences2graph.py --help

Type::

   python sequences2graph.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt

import CGAT.Experiment as E
import CGAT.Genomics as Genomics
import CGAT.Intervalls as Intervalls
import CGAT.PredictionParser as PredictionParser
import alignlib

param_long_options=["verbose=", "help", "version"]
param_short_options="v:ho:t:"

param_loglevel = 1

param_overlap_residues = 90
param_filename_filter_tokens = None

param_gop = -10.0
param_gep = -2.0

##------------------------------------------------------------
if __name__ == '__main__':

    try:
        optlist, args = getopt.getopt(sys.argv[1:], param_short_options, param_long_options)
    except getopt.error, msg:
        print globals()["__doc__"], msg
        sys.exit(2)
                                                                                                                                                             
    for o,a in optlist:
        if o in ( "-v", "--verbose" ):
            param_loglevel = int(a)
        elif o in ( "--version", ):
            print "version="
            sys.exit(0)
        elif o in ( "-h", "--help" ):
            print globals()["__doc__"]
            sys.exit(0)

    alignator = alignlib.makeAlignatorDPFull( alignlib.ALIGNMENT_LOCAL, param_gop, param_gep )
    map_query2token = alignlib.makeAlignmentVector()
    
    for line in sys.stdin:
        if line[0] == "#": continue

        query_token, sbjct_token, query_sequence, sbjct_sequence = string.split(line[:-1], "\t")

        map_query2token.clear()
        row = alignlib.makeSequence(query_sequence)
        col = alignlib.makeSequence(sbjct_sequence)
        alignator.align( map_query2token, row, col )

        pidentity = 100.0 * alignlib.calculatePercentIdentity( map_query2token, row, col )
        psimilarity = 100.0 * alignlib.calculatePercentSimilarity( map_query2token )        
        print string.join( map(str, (
            query_token, sbjct_token,
            map_query2token.getScore(),
            alignlib.AlignmentFormatEmissions( map_query2token ),
            pidentity,
            psimilarity,
            map_query2token.getNumGaps()) ), "\t" )
            
            
            
        



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
bed2annotator.py - convert bed to annotator format
==================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script converts a bed file into annotator compatible regions. Depending on the option --section
this script will create:

   segments
      a segments file

   annotations
      a file with annotations. Each bed track is a separate annotation.

   workspace
      a file with a workspace

Usage
-----

Example::

   python bed2annotator2tsv.py --help

Type::

   python bed2annotator2tsv.py --help

for command line help.

Documentation
-------------

Code
----

'''
import sys
import string
import re
import optparse
import time
import os
import shutil
import tempfile
import math
import itertools
import glob
import collections

import CGAT.Experiment as E
import CGAT.Bed as Bed
import CGAT.IndexedFasta as IndexedFasta
import CGAT.IOTools as IOTools

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: bed2annotator2tsv.py 2885 2010-04-07 08:46:50Z andreas $", 
                                    usage = globals()["__doc__"])
        
    parser.add_option( "-g", "--genome-file", dest="genome_file", type="string",
                       help="filename with genome."  )

    parser.add_option( "-f", "--features", dest="features", type="string", 
                       help="feature to collect [default=None]."  )

    parser.add_option( "-i", "--files", dest="files", action="append",
                       help="use multiple annotations [default=None]."  )

    parser.add_option(  "-a", "--annotations", dest="annotations", type="string", 
                       help="aggregate name for annotations if only single file is provided from STDIN [default=None]."  )

    parser.add_option(  "--input-filename-map", dest="input_filename_map", type="string", 
                       help="filename with a map of gene_ids to categories [default=None]."  )

    parser.add_option( "-l", "--max-length", dest="max_length", type="string", 
                       help="maximum segment length [default=None]."  )

    parser.add_option( "-m", "--merge", dest="merge", action="store_true",
                       help="merge overlapping bed segments [default=%default]."  )

    parser.add_option( "-s", "--section", dest="section", type="choice", 
                       choices=("segments", "annotations", "workspace" ),
                       help="annotator section [default=None]."  )

    parser.add_option( "--subset", dest="subsets", type="string", action="append",
                       help="add filenames to delimit subsets within the gff files. The syntax is filename.gff,label,filename.ids [default=None]."  )

    parser.set_defaults(
        genome_file = None,
        feature = None,
        remove_random = True,
        section = "segments",
        annotations = "annotations",
        max_length = 100000,
        files = [],
        subsets = [],
        input_filename_map = None,
        merge = False,
        )

    (options, args) = E.Start( parser )

    options.files += args
    if len(options.files) == 0: options.files.append("-")
    options.files = list( itertools.chain( *[ re.split( "[,; ]+", x) for x in options.files ] ) )

    if options.subsets:
        subsets = collections.defaultdict( list )
        for s in options.subsets: 
            filename_gff,label,filename_ids = s.split( "," )
            subsets[filename_gff].append( (label,filename_ids) )
        options.subsets = subsets

    if options.genome_file:
        fasta = IndexedFasta.IndexedFasta( options.genome_file )
    else:
        fasta = None

    if options.section == "segments":
        prefix = "##Segs"
    elif options.section == "annotations":
        prefix = "##Id"
    elif options.section == "workspace":
        prefix = "##Work"
    else:
        raise ValueError("unknown section %s" % options.section)

    if options.max_length:
        max_length = options.max_length
    else:
        max_length = 0

    ninput, ntracks, ncontigs, nsegments, ndiscarded = 0, 0, 0, 0, 0

    if options.section in ("annotations"):
        contigs = set()
        it = itertools.groupby( Bed.iterator( options.stdin ), key=lambda x: x.track["name"])

        map_track2segments = {}
        for track, beds in it:
            ntracks += 1
            map_track2segments[track] = []
            first_segment = nsegments

            beds = list(beds)

            if options.merge: beds = Bed.merge( beds )

            for bed in beds:
                contig, start, end = bed.contig, bed.start, bed.end

                if options.remove_random and "random" in contig: continue
                
                if max_length > 0 and end - start > max_length:
                    ndiscarded += 1
                    continue

                contigs.add(contig)
                map_track2segments[track].append( nsegments )
                options.stdout.write( "%s\t%i\t%s\t(%i,%i)\n" % (prefix, nsegments, contig, start, end ) )
                nsegments += 1
                
            options.stdout.write("##Ann\t%s\t%s\n" % (track, "\t".join( ["%i" % x for x in range(first_segment, nsegments) ] ) ) )
            E.info( "track %s: annotated with %i segments" % (track, nsegments - first_segment) )

        ncontigs = len(contigs)
        E.info( "ninput=%i, ntracks=%i, ncontigs=%i, nsegments=%i, ndiscarded=%i" % (ninput, ntracks, ncontigs, nsegments, ndiscarded))

    E.Stop()






################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
MatrixTools.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import sys, re, string, collections
import numpy
import pandas
import IOTools

class Matrix:

    def __init__(self):
        pass
    
def addOptions( parser ):
    """add matrices to option parser."""

    parser.add_option("-f", "--format", dest="format", type="string",
                      help="format."  )

    parser.add_option( "--row-names", dest="row_names", type="string",
                      help="list of row names separated by ','."  )
    
    parser.add_option( "--col-names", dest="col_names", type="string",
                      help="list of col names separated by ','."  )

    parser.add_option( "--file-row-names", dest="file_row_names", type="string",
                      help="filename with row names."  )
    
    parser.add_option( "--file-col-names", dest="file_col_names", type="string",
                      help="filename with col names."  )

    parser.add_option( "--numeric", dest="numeric", action="store_true",
                      help="row and column titles are numeric."  )

    parser.add_option( "--asymmetric", dest="asymmetric", action="store_true",
                      help="matrix is asymmetric."  )

    parser.add_option( "--default", dest="default", type="string",
                      help="default value for missing values."  )

    parser.add_option( "--default-diagonal", dest="default_diagonal", type="string",
                      help="default value for missing values on diagonal."  )

    parser.add_option( "--input-format", dest="input_format", type="choice",
                      choices=("row-col-weight", "", "row-col-weight-replicates"),
                      help="input format."  )

    parser.set_defaults(
        default = "0",
        default_diagonal = "0",
        format = "string",
        asymmetric = False,
        is_numeric = False,
        row_names = None,
        col_names = None,
        file_row_names = None,
        file_col_names = None,
        input_format = "row-col-weight",
        )

    
def getMapTokens( options ):

    map_token2row, map_token2col = {}, {}
    
    if options.file_row_names:
        row_tokens = map( lambda x: string.split(x[:-1], "\t")[0], open(options.file_row_names, "r").readlines())
        for row_token in row_tokens:
            map_token2row[row_token] = len(map_token2row)

    if options.row_names:
        for x in options.row_names.split(","):
            map_token2row[x] = len(map_token2row)

    if options.file_col_names:
        col_tokens = map( lambda x: string.split(x[:-1], "\t")[0], open(options.file_col_names, "r").readlines())
        for col_token in col_tokens:
            map_token2col[col_token] = len(map_token2col)

    if options.col_names:
        for x in options.col_names.split(","):
            map_token2col[x] = len(map_token2col)

    if not options.asymmetric:
        if map_token2row and not map_token2col:
            map_token2col = map_token2row
        elif map_token2col and not map_token2row:
            map_token2row = map_token2col

    return map_token2row, map_token2col

def getMatrixFromEdges( lines, options, in_map_token2row = {}, in_map_token2col = {}):
    """read matrix from lines
    """

    # remove comments
    lines = filter(lambda x:x[0] != "#" and len(x[:-1]) > 0, lines)

    if in_map_token2row:
        map_token2row = in_map_token2row
    else:
        map_token2row = {}

    if in_map_token2col:
        map_token2col = in_map_token2col
    else:
        map_token2col = {}

    if options.format == "string":

        has_row_names = len(map_token2row) > 0
        has_col_names = len(map_token2col) > 0
        
        ## if either row/column names are not given:
        if not map_token2row or not map_token2col:

            row_tokens = map(lambda x: string.split(x[:-1], "\t")[0], lines )
            col_tokens = map(lambda x: string.split(x[:-1], "\t")[1], lines )

            if options.is_numeric:
                row_tokens = map(float, row_tokens)
                col_tokens = map(float, col_tokens)                    
                row_tokens.sort()
                col_tokens.sort()
                row_tokens = map(str, row_tokens)
                col_tokens = map(str, col_tokens)                    
            else:
                row_tokens.sort()
                col_tokens.sort()

            if not has_row_names:
                for row_token in row_tokens:
                    if row_token not in map_token2row:
                        map_token2row[row_token] = len(map_token2row)
            if not has_col_names:
                for col_token in col_tokens:
                    if col_token not in map_token2col:
                        map_token2col[col_token] = len(map_token2col)

        if not options.asymmetric:
            for col_token in map_token2col.keys():
                if col_token not in map_token2row:
                    map_token2row[col_token] = len(map_token2row)            
            map_token2col = map_token2row

        matrix = [ [ options.default for j in range(len(map_token2col))] for i in range(len(map_token2row)) ]

        if len(map_token2col) == len(map_token2row):
            for j in range(len(map_token2col)):
                matrix[j][j] = options.default_diagonal

        ## return matrix
        m = Matrix()

        if options.input_format == "row-col-weight":
            for line in lines:
                row_token, col_token, weight = string.split(line[:-1], "\t")[:3]
                matrix[map_token2row[row_token]][map_token2col[col_token]] = weight
                if not options.asymmetric:
                    matrix[map_token2col[col_token]][map_token2row[row_token]] = weight
            
        elif options.input_format == "row-col-weight-replicates":
            replicates = [ [ 0 for j in range(len(map_token2col))] for i in range(len(map_token2row)) ]
            for line in lines:
                row_token, col_token, weight, n = string.split(line[:-1], "\t")[:4]
                matrix[map_token2row[row_token]][map_token2col[col_token]] = weight
                replicates[map_token2row[row_token]][map_token2col[col_token]] = int(n)
                if not options.asymmetric:
                    matrix[map_token2col[col_token]][map_token2row[row_token]] = weight
                    replicates[map_token2col[col_token]][map_token2row[row_token]] = int(n)                     
            m.mReplicates = replicates
            
        col_tokens = map_token2col.items()
        col_tokens.sort( lambda x,y: cmp(x[1],y[1]))
        row_tokens = map_token2row.items()
        row_tokens.sort( lambda x,y: cmp(x[1],y[1]))

        m.mMatrix = matrix
        m.mMapRow2Token = row_tokens
        m.mMapCol2Token = col_tokens
        m.mMapToken2Row = map_token2row
        m.mMapToken2Col = map_token2col        

    return matrix

def buildMatrixFromLists( lists, dtype = numpy.float, default = None ):
    '''build a matrix from a list of lists.

    Each list is a list of tuples (row, value).
    The columns are given by order of the lists.

    Returns matrix, row_headers
    '''

    all_rows = collections.defaultdict()    
    for l in lists: all_rows.update( [ (x[0],0) for x in l ] )
    for x,v in enumerate(all_rows.iteritems()):
        all_rows[v[0]] = x
        
    matrix = numpy.zeros( (len(all_rows), len( lists) ), dtype = dtype )
    if default: matrix.fill(default)
    
    for col, l in enumerate(lists):
        for row, value in l:
            matrix[all_rows[row],col] = value
    return matrix, all_rows.keys()

########################################################################
def buildMatrixFromTables( infiles, column, column_header = 0, dtype = numpy.float, default = None ):
    '''build a matrix from a column called *column* in a series of input files.
   
    If column_value == None, the first column is taken as the name of the row.

    The columns are given by order of the input files.

    returns matrix, row_headers
    '''
    
    lists = []
    for infile in infiles:
        data = pandas.read_table( IOTools.openFile(infile) )
        lists.append( zip( list( data[column_header] ), list(data[column]) ) )
        
    return buildMatrixFromLists( lists, dtype = dtype, default = default )
        



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
nr2table.py - convert description in nr fasta file to table
===========================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This scripts converts headers in ncbi `nr` fasta files to
a tabular format.

Usage
-----

Example::

   python nr2table.py --help

Type::

   python nr2table.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import optparse
import math
import time
import tempfile
import subprocess


import CGAT.Experiment as E
import Bio
import CGAT.FastaIterator as FastaIterator

##------------------------------------------------------------
if __name__ == '__main__':

    parser = E.OptionParser( version = "%prog version: $Id: nr2table.py 2782 2009-09-10 11:40:29Z andreas $", usage = globals()["__doc__"])

    parser.set_defaults()

    (options, args) = E.Start( parser )
    
    iterator = FastaIterator.FastaIterator( sys.stdin )

    sequences = []
    
    ninput, noutput, nentries = 0, 0, 0

    options.stdout.write( "gid\tsrc\tacc\tprotid\tannotation\tspecies\n" )
    while 1:
        
        cur_record = iterator.next()

        if cur_record == None: break

        ninput += 1

        records = cur_record.title.split( chr(1) )
        for record in records:

            a, anno = re.search("(\S+)\s+(.+)", record).groups()

            vals = a.split("|")
            
            try:
                gi, gid, src, acc, protid = vals
            except ValueError:
                raise "parsing error for record: %s" % record
            
            try:
                annotation, species = re.search( "(.+)\s+\[(.*)\]", anno).groups()
            except AttributeError:
                annotation = anno.strip()
                species = ""

            annotation = re.sub( "\t", " ", annotation)

            options.stdout.write( "\t".join( (gid, src, acc, protid,
                                              annotation, species) ) + "\n" )

            nentries += 1
            
        noutput += 1
        
    if options.loglevel >= 1:
        options.stdlog.write( "# ninput=%i, noutput=%i, nentries=%i\n" % (ninput, noutput, nentries ))
        
    E.Stop()
    
    



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
list_overlap.py - compute overlap between lists
===============================================

:Author: 
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

taken from https://github.com/brentp/bio-playground/blob/master/utils/list_overlap_p.py
find the probability that as high as `shared_genes` is random
given the number of genes: `A_genes`, `B_genes` drawn from `total_genes`
e.g.:

$ %prog shared_genes total_genes A_genes B_genes

or:

$ %prog 10 30000 345 322
0.0043679470685

gives the probability that, 2 random gene subsets (chosen from 30000 genes)
of length 345 and 322 would share at least 10 genes by chance.


This can also be called with 3 files of gene_names:

$ %prog all_genes.txt A_genes.txt B_genes.txt

A_genes.txt and B_genes.txt are then intersected to get the numbers for
the hypergeometric test. Each file must be a single column containing
the gene-name. The comparison *is* case sensitive.


See: http://www.nslij-genetics.org/wli/pub/ieee-embs06.pdf

Usage
-----

Example::

   python cgat_script_template.py --help

Type::

   python cgat_script_template.py --help

for command line help.

Documentation
-------------

Code
----

'''

import optparse
import sys
import os.path as op
import scipy.stats as ss
import CGAT.Experiment as E

def hypergeom(m, n, n1, n2, p=False):
    """
>>> hypergeom(1, 1000, 1000, 1000) # has to be shared.
1.0

>>> all(hypergeom(i, 1000, 1000, 1000) == 1.0 for i in range(100))
True

>>> hypergeom(1, 30000, 20, 20)
0.013253396616299651

>>> hypergeom(2, 30000, 20, 20)
7.9649366037104485e-05

>>> hypergeom(11, 30000, 20, 20)
4.516176321800458e-11

>>> hypergeom(10, 30000, 20, 20) # very low prob.
4.516176321800458e-11

>>> hypergeom(20, 30000, 20, 20) # very low chance that all are shared.
4.516176321800458e-11

"""
    if m <= 0: return 1.0
    mmin = m - 1
    mmax = min(n1, n2)
    return ss.hypergeom.cdf(mmax, n, n1, n2) - ss.hypergeom.cdf(mmin, n, n1, n2)

def with_genes(fftot, ffa, ffb, asfile=True):
    """
    given 3 genelists, calculate the p-value of the shared
    genes between fa and fb that are drawn from ftot.
    """

    if asfile:
        ftot = frozenset(f.strip() for f in open(fftot) if f.strip())
        fa = frozenset(f.strip() for f in open(ffa) if f.strip())
        fb = frozenset(f.strip() for f in open(ffb) if f.strip())
    else:
        fa, fb, ftot = frozenset(ffa), frozenset(ffb), frozenset(fftot)
    
    n1, n2 = len(fa), len(fb)
    m = len(fa.intersection(fb))
    n = len(ftot)

    if asfile:
        print "A : %-32s:%-5i" % (ffa, n1)
        print "B : %-32s:%-5i" % (ffb, n2)
        print "total : %-32s:%-5i" % (fftot, n)
        print "shared: %-32s:%-5i" % (' ', m)
    else:
        print "A : %-32s:%-5i" % ("set A", n1)
        print "B : %-32s:%-5i" % ("set B", n2)
        print "total : %-32s:%-5i" % ("total", n)
        print "shared: %-32s:%-5i" % (' ', m)
   
    return hypergeom(m, n, n1, n2)


def main():
    p = E.OptionParser(__doc__,
                       version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", )
    opts, args = p.parse_args()
    if (len(args) not in (3, 4)):
        sys.exit(not p.print_help())
    if len(args) == 4 and not all(a.isdigit() for a in args):
        print >>sys.stderr, "four arguments must be integers"
        sys.exit(not p.print_help())
    elif len(args) == 3 and not all(op.exists(f) for f in args):
        sys.exit(not p.print_help())

    if len(args) == 4:
        args = map(long, args)
        m, n, n1, n2 = args
        result = hypergeom(m, n, n1, n2)
        #print type(result)
        print result
    else:
        tot_genes, a_genes, b_genes = map(str.strip, args)
        print with_genes(tot_genes, a_genes, b_genes)



if __name__ == "__main__":
    #import doctest
    #if doctest.testmod(optionflags=doctest.ELLIPSIS).failed == 0:
    #    main()
    main()




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: snp2maf.py 2875 2010-03-27 17:42:04Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
snp2map.py - build maf formatted multiple genomic alignments from SNPs
======================================================================

:Author: Andreas Heger
:Release: $Id: snp2maf.py 2875 2010-03-27 17:42:04Z andreas $
:Date: |today|
:Tags: Python

Purpose
-------

construct alleles of a reference sequences based on
variants in an sqlite database.

Caveats
-------

* no phasing
* the two allelic sequences should be built per genome

Usage
-----

Type::

   python <script_name>.py --help

for command line help.

Code
----

""" 

import os
import sys
import re
import optparse
import collections
import gzip
import sqlite3

import numpy
import CGAT.Experiment as E
import CGAT.IOTools as IOTools
import CGAT.IndexedFasta as IndexedFasta
import CGAT.Genomics as Genomics
import CGAT.GTF as GTF
import CGAT.Variants as Variants
import alignlib

def alignIndels( all_alleles, colcounts, extend_by = 0 ):
    '''align all indel-regions.'''

    aa = alignlib.makeAlignatorDPFull( alignlib.ALIGNMENT_LOCAL, 0, 0 )     
    alignator = alignlib.makeMultipleAlignatorSimple( aa)

    ids = all_alleles.keys()

    for x,c in enumerate(colcounts):
        if c <= 1: continue
        sequences = alignlib.StringVector()
        for sid in ids:
            for allele in all_alleles[sid]:
                sequences.append( allele[x] )

        mali = alignlib.makeMultAlignment()
        alignator.align( mali, sequences )
        realigned = []
        for line in str(alignlib.MultAlignmentFormatPlain( mali, sequences )).split("\n")[:-1]:
            data = line[:-1].split("\t")
            realigned.append( data[1] )
        assert len(realigned) == len(sequences)

        l = max( [len(r) for r in realigned] )
        i = 0
        for sid in ids:
            for allele in all_alleles[sid]:
                if realigned[i]: allele[x] = realigned[i] 
                else: allele[x] = "-" * l 
                i += 1
                
        colcounts[x] = l


def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: snp2maf.py 2875 2010-03-27 17:42:04Z andreas $", usage = globals()["__doc__"] )

    parser.add_option("-g", "--genome-file", dest="genome_file", type="string",
                      help="filename with genome [default=%default]."  )
    parser.add_option("-t", "--tracks", dest="tracks", type="string", action="append",
                      help="tracks (tablenames) to use in sqlite database [default=%default]."  )
    parser.add_option("-d", "--database", dest="database", type="string",
                      help="sqlite3 database [default=%default]."  )
    parser.add_option("-r", "--reference", dest="reference", type="string",
                      help="name of reference [default=%default]."  )
    parser.add_option("-i", "--is-gtf", dest="is_gtf", action="store_true",
                      help="if set, the gene_id will be added to the alignment header [default=%default]."  )
    parser.add_option("-z", "--compress", dest="compress", action="store_true",
                      help="compress output with gzip [default=%default]."  )
    parser.add_option("-p", "--pattern", dest="pattern_track", type="string",
                      help="regular expression pattern for track [default=%default].")

    parser.set_defaults(
            genome_file = None,
            tracks = [],
            database = "csvdb",
            output = [],
            border = 0,
            reference_name = "reference",
            pattern_track = "(\S+)",
            is_gtf = True,
            compress = False,
            )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv, add_output_options = True )

    ninput, nskipped, noutput = 0, 0, 0

    if not options.database or not options.tracks:
        raise ValueError("please supply both database and tracks")

    if options.genome_file:
        fasta = IndexedFasta.IndexedFasta( options.genome_file )
    else:
        fasta = None

    if options.is_gtf:
        infile_gff = GTF.iterator( options.stdin)
    else:
        infile_gff = GTF.iterator( options.stdin)

    dbhandle = sqlite3.connect( options.database )

    statement = '''SELECT pos, reference, genotype 
                   FROM %(track)s
                   WHERE contig = '%(contig)s' AND 
                   pos BETWEEN %(extended_start)s and %(extended_end)s
                '''

    counts = E.Counter()
    tracks = options.tracks
    try:
        translated_tracks = [ re.search( options.pattern_track, track).groups()[0] for track in tracks ]
    except AttributeError:
        raise AttributeError( "pattern `%s` does not match input tracks." % options.pattern_track )

    if options.compress:
        outfile = gzip.GzipFile( fileobj = options.stdout )
    else:
        outfile = options.stdout

    outfile.flush()
    outfile.write( "##maf version=1 program=snp2maf.py\n\n") 

    for gff in infile_gff:
        counts.input += 1
        
        contig = gff.contig
        strand = gff.strand
        lcontig = fasta.getLength( contig )
        region_start, region_end = gff.start, gff.end
        if contig.startswith("chr"): contig = contig[3:]
        extended_start = region_start - options.border
        extended_end = region_end + options.border
        is_positive = Genomics.IsPositiveStrand( strand )

        E.info("processing %s" % str(gff))

        # collect all variants
        all_variants = []
        for track in options.tracks:
            cc = dbhandle.cursor()
            cc.execute( statement % locals() )
            all_variants.append( map(Variants.Variant._make, cc.fetchall()) )
            cc.close()
            
        E.debug("%s:%i..%i collected %i variants for %i tracks" % (contig, 
                                                                   region_start, region_end, 
                                                                   sum( [len(x) for x in all_variants] ),
                                                                   len(all_variants)))

        reference_seq = fasta.getSequence( contig, "+", region_start, region_end ) 
        lseq = len(reference_seq)
        alleles = collections.defaultdict( list )
            
        # build allele sequences for track and count maximum chars per mali column
        colcounts = numpy.ones( lseq )
        for track, variants in zip(translated_tracks, all_variants):
            variants = Variants.updateVariants( variants, lcontig, "+" )
            a = Variants.buildAlleles(reference_seq, 
                                      variants, 
                                      reference_start = region_start )
            
            alleles[track] = a
            for allele in a:
                for pos, c in enumerate( allele):
                    colcounts[pos] = max(colcounts[pos], len(c))

        # realign gapped regions
        alignIndels( alleles, colcounts ) 
        
        if options.is_gtf:
            outfile.write( "a gene_id=%s\n" % gff.gene_id )
        else:
            outfile.write( "a\n" )

        maf_format = "s %(name)-30s %(pos)9i %(size)6i %(strand)s %(lcontig)9i %(seq)s\n" 
        def __addGaps( sequence, colcounts ):
            '''output gapped sequence.'''
            r = []
            for x,c in enumerate(sequence):
                r.append( c + "-" * (colcounts[x]-len(c)))
            return "".join(r)

        name = ".".join((options.reference, contig))
        if is_positive:
            pos = region_start
        else:
            pos = lcontig - region_start

        size = lseq
        seq = __addGaps( reference_seq, colcounts )
        outfile.write( maf_format % (locals()) )
        
        for track in translated_tracks:
            for aid, allele in enumerate(alleles[track]):
                seq = __addGaps( allele, colcounts )
                if not is_positive: Genomics.complement( seq )
                size = len(seq) - seq.count("-")
                name = ".".join( (track + "-%i" % aid, contig))
                outfile.write( maf_format % (locals()) )
                
        outfile.write("\n")

    E.info( "%s" % str(counts))

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
cgat_build_report_page.py - build report page for all projects
=======================================================================

:Author: 
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script scans all of :file:`/ifs/projects/sftp` for :file:`index.html` files 
and outputs an html formatted summary table into :file:`/ifs/projects/overview`.

Usage
-----

Example::

   python cgat_build_report_page.py 

Type::

   python cgat_build_report_page.py --help

for command line help.

Documentation
-------------

Code
----

'''

import os
import sys
import re
import optparse
import subprocess

import CGAT.Experiment as E
import CGAT.IOTools as IOTools

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option("-p", "--path", dest="path", type="string",
                      help="path to scan for files [%default]"  )

    parser.add_option("-d", "--destination", dest="destination", type="string",
                      help="path to deposit files into [%defaul]"  )

    parser.set_defaults( path = '/ifs/projects/sftp',
                         url = 'http://www.cgat.org/downloads/',
                         dest = '/ifs/projects/overview' )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    statement = "find %s -name 'index.html'" % options.path

    process = subprocess.Popen(  statement,
                                 shell = True,
                                 stdin = subprocess.PIPE,
                                 stdout = subprocess.PIPE,
                                 stderr = subprocess.PIPE )

    stdout, stderr = process.communicate()

    files = stdout.split('\n')
    files.sort()

    outfile = IOTools.openFile( os.path.join( options.dest, "index.html" ), "w" )

    outfile.write( '''
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">

<html>
<head>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
    <title>CGAT project reports</title>
    <link rel="stylesheet" href="cgat.css" type="text/css" />
    <link rel="stylesheet" href="pygments.css" type="text/css" />
    <link rel="shortcut icon" href="http://cgatwiki.anat.ox.ac.uk/favicon.ico">
    <script type="text/javascript" src="sorttable.js"></script>
</head>

  <body>
    <div class="related">
      <h3>Navigation</h3>
      <ul>
        <li><a href="index.html">CGAT Projects Overview</a> &raquo;</li>
      </ul>
    </div>

    <div class="document">
      <div class="documentwrapper">
        <div class="bodywrapper">
          <div class="body">
 <div class="section" id="cgat-pipelines">
<H1>CGAT exported project pages</H1>

<p>
This page is for internal use only. Do not distribute outside of CGAT and 
do not make this page available on the world wide web.
</p>

<table class="sortable">\n''' )

    outfile.write( '''<tr><th>Project</th><th>Report</th><th>Title</th></tr>\n''' )

    for f in files:
        if f == '': continue

        proj = re.search( '(proj\d+)', f ).groups()[0]
        relpath = re.sub( '.*proj\d+/', '', f )
        report = re.sub( '^[^/]*/', '', os.path.dirname( relpath ) )

        lines = IOTools.openFile( f ).readlines()
        titles = [ x for x in lines if "<title>" in x ]
        if titles:
            title = re.search( "<title>(.*)</title>", titles[0] ).groups()[0]
        else:
            title = "NA"

        if title.endswith("documentation"):
            title = title[:-len("documentation")]

        url = os.path.join( options.url, relpath )
        outfile.write( '<tr><td>%(proj)s</td><td><a HREF="%(url)s">%(report)s</td><td>%(title)s</td></tr>\n' % locals() )
                              
    outfile.write( '''
</table>

</div>
</div>


          </div>
        </div>
      </div>
      <div class="sphinxsidebar">
        <div class="sphinxsidebarwrapper">
            <p class="logo"><a href="contents.html">
              <img class="logo" src="cgat_logo.png" alt="Logo"/>
            </a></p>





</body>
</html>\n''' )

    outfile.close()

    E.info( 'created output file %s' % outfile.name)
    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
WrapperMuscle.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import os, sys, string, re, tempfile, subprocess, optparse, shutil

from types import *

"""Wrapper for Muscle
"""

import Experiment
import Mali

class Error(Exception):
    """Base class for exceptions in this module."""
    pass

    def __str__(self):
        return str(self.message)

class ParsingError(Error):
    """Exception raised for errors while parsing

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message, line):
        self.message = message + " at line " + line

class UsageError(Error):
    """Exception raised for errors while starting

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message):
        self.message = message

class Muscle:

    mExecutable = "muscle -maxmb 10000"
    
    def __init__( self ):
        
        self.mTempdir = None
        
    def __del__(self):

        if self.mTempdir and self.mTempdir != "tmp":
            os.system( "rm -rf %s" % self.mTempdir )

    ##------------------------------------------------------------------------
    def Run( self, mali,
             tree = None,
             dump = 0,
             test = False,
             options = {} ):

        self.mTempdir = tempfile.mkdtemp()
        self.mFilenameInput = "input"
        self.mFilenameOutput = "output"        
        
        if test:
            print "# temporary directory is %s" % self.mTempdir

        mali.writeToFile( open(self.mTempdir + "/" + self.mFilenameInput, "w"),
                          format="fasta" )

        statement = " ".join( (self.mExecutable,
                               "-in %s" % self.mFilenameInput,
                               "-out %s" % self.mFilenameOutput ) )
                              
        s = subprocess.Popen( statement,
                              shell = True,
                              stdout = subprocess.PIPE,
                              stderr = subprocess.PIPE,
                              cwd = self.mTempdir,
                              close_fds = True)                              

        (out, err) = s.communicate()

        if s.returncode != 0:
            raise UsageError, "Error in running %s \n%s\n%s\nTemporary directory in %s" % (self.mExecutable, err, out, self.mTempdir)

        if dump:
            print "# stdout output of %s:\n%s\n######################################" % (self.mExecutable, out)

        result = Mali.Mali()

        result.readFromFile( open( "%s/%s" % (self.mTempdir, self.mFilenameOutput), "r"), format= "fasta")

        if not test:
            shutil.rmtree( self.mTempdir )

        return result

if __name__ == "__main__":


    parser = E.OptionParser( version = "%prog version: $Id: WrapperMuscle.py 2784 2009-09-10 11:41:14Z andreas $" )

    parser.add_option("-t", "--filename-input-tree", dest="filename_input_tree", type="string",
                      help="filename with tree information."  )
    parser.add_option("-T", "--filename-output-tree", dest="filename_output_tree", type="string",
                      help="output filename with tree information."  )
    parser.add_option("-p", "--program", dest="program", type="string",
                      help="program to use."  )
    parser.add_option("-o", "--options", dest="options", type="string",
                      help="input options."  )

    parser.set_defaults(
        filename_input_tree = None,
        filename_output_tree = None,
        program = None,
        options = "",
        )

    (options, args) = Experiment.Start( parser )

    Experiment.Stop()
    



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
bam2UniquePairs.py - filter/report uniquely mapped read pairs from a (bwa!) bam-file
====================================================================================

:Author: Steve Sansom
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

Simply utitily script to report and/or filter out "uniquely mapped" properly paired reads

Reports:
1. The percentage of properly mapped read pairs with at least one uniquely mapped (XT=U) read
2. The percentage of properly mapped read pairs with at least one best mapped (X0-1) read
3. The percentage of properly mapped read pairs with at least one uniquely or best mapped (X0-1) read

If outfile is specified, reads are emitted when they are properly paired and the pair has at least one read that is either best or uniquely mapped.

Duplication is ignored.

Only BWA is supported.

TODO: cache and emit reads rather than iterating over the samfile twice...

'''

import os
import sys
import re
import optparse
import collections
import CGAT.Experiment as E
import CGAT.IOTools as IOTools
import pysam


def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option( "-f", "--filename", dest="filename", type="string",
                       help = "bamfile")

    parser.add_option( "-a", "--aligner", dest="aligner", type="string",
                       help = "bamfile", default="bwa")

    parser.add_option( "-r", "--report", type="string", dest="report", 
                   help = "bamfile", default="")
   
    parser.add_option( "-o", "--outfile", dest="outfile", type="string",
                       help = "bamfile", default="")
                       

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv, add_output_options = True )

    # Check the aligner is supported
    if options.aligner != "bwa":
        raise ValueError("Currently only bwa is supported as aligner specific flags are used")

    # Check that either a report or outfile name has been specified
    if options.report == "" and options.outfile == "":
        raise ValueError("Nothing to do")

    # Analyse the bamfile
    samfile = pysam.Samfile(options.filename,"rb")
    uniq_map, best_map, uORb_map = {}, {}, {}
    properly_paired = 0

    for read in samfile.fetch():

        if read.is_proper_pair:
            tagd = dict(read.tags)
            u, b, key = False, False, read.qname

            if tagd["XT"] == "U":
                u = True
                uniq_map[key]=1

            if "X0" in tagd:
                if tagd["X0"] == 1:
                    b = True
                    best_map[key]=1

            if u == True or b == True:
                uORb_map[key] = 1
                
            properly_paired += 1

    samfile.close()

    npp = properly_paired/2

    E.info( "No proper pairs: %s" % npp)

    # Write a tabular report if report name given
    if options.report != "":
        
        E.info( "Writing report on no. proper pairs with unique/best reads")
        
        def _row(x, npp=npp):
            name, d = x
            n = len(d.keys())
            pc = float(n)/npp * 100
            line = "%s\t%i\t%.2f" % (name, n, pc)
            return(line)

        header = "\t".join(["pair_criteria","n_proper_pairs",
                            "percent_proper_pairs"])

        with open(options.report, "w") as report:
            report.write( header + "\n")
            for x in [("unique", uniq_map), ("best", best_map),
                      ("unique_or_best", uORb_map) ]:
                report.write(_row(x) + "\n")

    # Create new bam containing uniquely mapping read pairs
    # if outfile specified
    if options.outfile != "":

        E.info("Writing proper pairs with unique or best read to %s" % options.outfile)

        samfile = pysam.Samfile(options.filename,"rb")
        outbam = pysam.Samfile(options.outfile, "wb", template=samfile)

        for read in samfile.fetch():
            if read.is_proper_pair:
                if read.qname in uORb_map:
                    outbam.write(read)
        samfile.close()
        outbam.close()
        
if __name__ == "__main__":
    sys.exit( main( sys.argv) )



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: psl2chain.py 2901 2010-04-13 14:38:07Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
psl2chain.py - convert psl formatted file to a chain formatted file
===================================================================

:Author: Andreas Heger
:Release: $Id: psl2chain.py 2901 2010-04-13 14:38:07Z andreas $
:Date: |today|
:Tags: Python

Purpose
-------

convert a `psl <http://genome.ucsc.edu/FAQ/FAQformat.html#format2>`_
formatted file to a `chain <http://www.breyer.com/ucsc/htdocs/goldenPath/help/chain.html>`_ 
formatted file for use in liftover.

The nomenclature the UCSC uses is :file:`TargetToQuery.chain` for mapping ``target``
to ``query`` (according to the UCSC documentation, ``target`` is the first entry
in ``chain`` files). I have been using the nomenclature ``QueryToTarget.psl``. Hence,
the correct way to converting a psl file is::

   python psl2chain.py < queryToTarget.psl > targetToQuery.chain

Usage
-----

Type::

   python <script_name>.py --help

for command line help.

Code
----

""" 

import os
import sys
import re
import optparse

import CGAT.Experiment as E
import CGAT.Blat as Blat

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: psl2chain.py 2901 2010-04-13 14:38:07Z andreas $", 
                                    usage = globals()["__doc__"] )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    ## do sth
    ninput, nskipped, noutput = 0, 0, 0

    for psl in Blat.iterator(options.stdin):
        ninput += 1
        if psl.strand == "-":
            qstart, qend = psl.mQueryLength - psl.mQueryTo, psl.mQueryLength - psl.mQueryFrom
        else:
            qstart, qend = psl.mQueryFrom, psl.mQueryTo

        options.stdout.write( "chain %i %s %i %s %i %i %s %i %s %i %i %i\n" % \
                                  (psl.mNMatches, 
                                   psl.mSbjctId,
                                   psl.mSbjctLength,
                                   "+",
                                   psl.mSbjctFrom,
                                   psl.mSbjctTo,
                                   psl.mQueryId,
                                   psl.mQueryLength,
                                   psl.strand,
                                   qstart,
                                   qend,
                                   ninput ) )
            
        size, tend, qend = 0, None, None
        for qstart, tstart, size in psl.getBlocks():
            if tend != None:
                options.stdout.write( "\t%i\t%i\n" % (tstart - tend, qstart - qend) )
            qend, tend = qstart + size, tstart + size
            options.stdout.write( "%i" % (size,))
        options.stdout.write( "\n" )

        noutput += 1

    E.info( "ninput=%i, noutput=%i, nskipped=%i" % (ninput, noutput,nskipped) )

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
SequencePairProperties.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import numpy
import WrapperCodeML
import Mali

class SequencePairProperties:

    mPseudoCounts = 1
    
    def __init__(self ):

        self.mLength = 0
        self.mNCodons = 0
        
    def addProperties( self, other ):
        """add properties."""

        self.mLength += other.mLength
        self.mNCodons += other.mNCodons

    def updateProperties( self ):
        pass
    
    def loadPair( self, seq1, seq2 ):
        """load sequence properties from a pair."""
        pass
        
    def __str__(self):
        return "\t".join( self.getFields() )

    def __call__(self, seq1, seq2):
        self.loadPair( seq1.upper(), seq2.upper())

    def getFields( self ):
        self.updateProperties()        
        return []
    
    def getHeaders( self ):
        return []

    def getHeader(self):
        return "\t".join( self.getHeaders )

class SequencePairPropertiesDistance(SequencePairProperties):
    """base class for distance estimators."""
    def __init__(self, *args, **kwargs ):
        SequencePairProperties.__init__(self, *args, **kwargs )

class SequencePairPropertiesBaseML (SequencePairPropertiesDistance):
    """Counts for nucleic acid sequences.
    
    The first characters are ACGT.
    """

    mFormat = "%6.4f" 

    def __init__(self, options, *args, **kwargs ):
        SequencePairPropertiesDistance.__init__(self, *args, **kwargs )
        
        self.mBaseml = WrapperCodeML.BaseML()
        self.mBaseml.SetOptions( options )

        if options.loglevel >= 3:
            self.mDump = True
            self.mTest = True
        else:
            self.mDump = False
            self.mTest = False

    def loadPair( self, seq1, seq2 ):

        temp_mali = Mali.Mali()
        temp_mali.addSequence( "seq1", 0, len(seq1), seq1 )
        temp_mali.addSequence( "seq2", 0, len(seq2), seq2 )

        try:
            self.mResult = self.mBaseml.Run( temp_mali,
                                             tree = "(seq1,seq2);",
                                             dump = self.mDump,
                                             test = self.mTest )
        except WrapperCodeML.UsageError:
            self.mResult = None

    def getHeaders(self):
        return ["distance", "lnl", "kappa", "converged"]

    def __str__(self):

        if not self.mResult:
            return "\t".join( ["na"] * 4  )

        if self.mResult.mKappa != "na":
            o_kappa = self.mFormat % self.mResult.mKappa 
        else:
            o_kappa = self.mResult.mKappa 

        if self.mResult.mCheckConvergence:
            o_converged = "0"
        else:
            o_converged = "1"

        return "\t".join( (
                self.mFormat % self.mResult.mDistanceMatrix["seq1"]["seq2"],
                self.mFormat % self.mResult.mLogLikelihood ,
                o_kappa,
                o_converged ) )

class SequencePairPropertiesCountsNa (SequencePairProperties):
    """Counts for nucleic acid sequences.
    
    The first characters are ACGT.
    """

    mAlphabet = "ACGT"
    mGapChar = "-"

    def __init__(self, *args, **kwargs ):
        SequencePairProperties.__init__(self, *args, **kwargs )
        self.mNIdentical = 0
        self.mNAligned = 0
        self.mNDifferent = 0
        self.mNTransitions = 0
        self.mNTransversions = 0
        self.mNUnaligned1 = 0
        self.mNUnaligned2 = 0        
        self.mMapChar2Pos = {}
        self.mPercentGC = 0
        self.mMatrix = []

    def buildSubstitutionMatrix( self, seq1, seq2, alphabet ):

        """given a pair of sequences, calculate
        a substitution matrix for the given alphabet.
        """
        if len(seq1) != len(seq2):
            raise "two sequences of unequal length submitted."

        nchars = len(alphabet)
        matrix = numpy.zeros( (nchars,nchars), numpy.int)
        map_char2pos = {}
        for x in alphabet: 
            map_char2pos[x] = len(map_char2pos)
        
        for x in range(len(seq1)):
            try:
                matrix[map_char2pos[seq1[x]],map_char2pos[seq2[x]]] += 1
            except KeyError:
                continue

        return matrix
    
    def loadPair( self, seq1, seq2 ):
        """load sequence properties from a pair.
        """
        SequencePairProperties.loadPair( self, seq1, seq2 )

        alphabet = self.mAlphabet+self.mGapChar
    
        map_char2pos = {}
        for x in alphabet: 
            map_char2pos[x] = len(map_char2pos)
            
        # build coordinates for various substitution subsets
        transitions, transversions = [], []
        for x in ("AG", "GA", "CT", "TC"):
            transitions.append( (map_char2pos[x[0]], map_char2pos[x[1]]) )

        for x in ("AT", "TA", "GT", "TG", "GC", "CG", "AC", "CA" ):
            transversions.append( (map_char2pos[x[0]], map_char2pos[x[1]]) )

        matrix = self.buildSubstitutionMatrix( seq1, seq2, alphabet )
        matrix_acgt = matrix[0:4,0:4]

        self.mMatrix = matrix
        self.mMapChar2Pos = map_char2pos
        self.mNAligned = numpy.sum(numpy.ravel(matrix_acgt) )
        self.mNIdentical = numpy.sum( numpy.trace(matrix_acgt) )
        self.mNTransitions = numpy.sum( [ matrix[x] for x in transitions ] )
        self.mNTransversions = numpy.sum( [ matrix[x] for x in transversions ] )
        self.mNDifferent = self.mNAligned - self.mNIdentical
        self.mNUnaligned1 = numpy.sum(numpy.ravel(matrix[0:4,4]))    
        self.mNUnaligned2 = numpy.sum(numpy.ravel(matrix[4,0:4]))    

        cp = self.mMapChar2Pos['C']
        gp = self.mMapChar2Pos['G']

        # sum all rows and columns that have a least one G or C
        # and remove those that have two
        gc = numpy.sum(
             self.mMatrix[0:4,cp] \
              + self.mMatrix[0:4,gp] \
              + self.mMatrix[cp,0:4] \
              + self.mMatrix[gp,0:4]) \
              - self.mMatrix[cp,cp] \
              - self.mMatrix[gp,gp] \
              - self.mMatrix[cp,gp] \
              - self.mMatrix[gp,cp] 

        try:
             self.mPercentGC = "%5.2f" % (100.0 * float(gc) / self.mNAligned)
        except ZeroDivisionError:
            self.mPercentGC = "na"

    def __str__(self):
        
        return "\t".join(map(str, (self.mNIdentical, self.mNAligned, self.mNDifferent,
                                   self.mNTransitions, self.mNTransversions,
                                   self.mNUnaligned1, self.mNUnaligned2,
                                   self.mPercentGC)) )

    def getHeaders( self ):
        return ["identical", "aligned", "different", "transitions", "transversions",
                "unaligned1", "unaligned2", "pgc" ]

class SequencePairPropertiesCountsCodons( SequencePairPropertiesCountsNa):
    """the first characters are ACGT."""
    def __init__(self):
        SequencePairPropertiesCountsNa.__init__(self)
        self.mNNonSynonymous = 0
        self.mNSynonymous = 0

    def __str__(self):

        return "\t".join( (SequencePairPropertiesCountsNa.__str__(self),
                           str(self.mNNonSynonymous), 
                           str(self.mNSynonymous )))

    def getHeaders( self ):
        return ["nonsyn", "nsyn"]


class SequencePairPropertiesPID( SequencePairPropertiesDistance):

    """the first characters are ACGT."""
    mGapChars = ("-", "."),

    def __init__(self, *args, **kwargs):
        SequencePairPropertiesDistance.__init__(self, *args, **kwargs )
        self.mPID = 0

    def loadPair( self, seq1, seq2 ):

        nidentical, naligned, nunaligned = 0, 0, 0
        for c1, c2 in zip( seq1.upper(), seq2.upper()):
            if c1 in self.mGapChars or c2 in self.mGapChars:
                continue
            naligned += 1        
            if c1 == c2:
                nidentical += 1

        self.mPID = 100.0 * float(nidentical) / naligned

    def getHeaders(self):
        return ["distance",]

    def __str__(self):
        return self.mFormat % self.mPID

        



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
WrapperNJTree.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import os, sys, string, re, tempfile, subprocess, optparse, shutil

from types import *

"""Wrapper for NJTree
"""

import Experiment as E
import Mali
import IOTools
import TreeTools

class Error(Exception):
    """Base class for exceptions in this module."""
    pass

    def __str__(self):
        return str(self.message)

class ParsingError(Error):
    """Exception raised for errors while parsing

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message, line):
        self.message = message + " at line " + line

class UsageError(Error):
    """Exception raised for errors while starting

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message):
        self.message = message

class Result:
    
    mMaxLineLength = 100
    
    def __init__(self):
        self.mLog = None
        self.mErr = None
        
    def __str__(self):
        return TreeTools.Tree2Newick(self.mTree)
        
class IdentifierParser:

    def __init__(self, map_species2sp = None):
        self.mMapSpecies2Sp = map_species2sp

class IdentifierParserGPipe(IdentifierParser):

    mSeparator = "|"
    
    def __init__(self, *args, **kwargs):
        IdentifierParser.__init__(self, *args, **kwargs)

    def __call__(self, id ):
        data = id.split( self.mSeparator )

        species = data[0]
        
        if len(data) == 2:
            gene = data[1]
            transcript = None
        elif len(data) >= 3:
            gene = data[2]
            transcript = data[1]
            
        if self.mMapSpecies2Sp:
            species = self.mMapSpecies2Sp[species]

        return species, gene, transcript

class NJTree:
    """
    """
    mExecutable = "treebest"

    def __init__( self, identifier_parser ):

        self.mFilenameSpeciesTree = None
        self.mFilenameMali = None
        self.mFilenameTree = None
        
        self.mIdentifierParser = identifier_parser

        self.mDefaults = { "positive_only": "0",
                           "initial_kappa": "2.0",
                           "initial_omega": "0.1" }

        self.mOptions = {
            "positive_only" : {  "0": "output positve and neutral sites",
                                 "1": "only output positive sites"
                                 },
            "initial_kappa"   : { "#" : "initial value for kappa",
                                  },
            "initial_omega"  : { "#" : "initial value for omega",
                                 },
            }

        self.mLog = sys.stdout
        self.mErr = sys.stderr

    ##------------------------------------------------------------------------
    def SetLog( self, log ):
        """set log stream."""
        self.mLog = log

    ##------------------------------------------------------------------------
    def SetErr( self, err ):
        """set error stream."""
        self.mErr = err
        
    ##------------------------------------------------------------------------    
    def GetOptions( self ):
        """return options in pretty format"""
        result = ["# Options for NJTree" ]
        for var in self.mOptions.keys():
            result.append("# %-40s: %s" % (var, self.mOptions[var]))
        return string.join(result, "\n")


    ##------------------------------------------------------------------------
    def WriteAlignment( self, mali ):
        """write alignment in Phylip format."""

        mali.mapIdentifiers( self.mMapOld2New )
        outfile = open( self.mTempdir + "/" + self.mFilenameMali, "w" )
        mali.writeToFile( outfile, format="plain-fasta" )
        outfile.close()

    ##------------------------------------------------------------------------
    def BuildMap( self, mali ):
        """build a map of identifiers that conform to the NJ standard."""

        self.mMapOld2New = {}
        self.mMapNew2Old = {}
        for old_id in mali.getIdentifiers():

            species, gene, transcript = self.mIdentifierParser( old_id )

            if transcript:
                new_id = "%s_%s GENEid=%s" % ( transcripts, species, gene )
            else:
                new_id = "%s_%s" % ( gene, species )                

            self.mMapOld2New[old_id] = new_id
            self.mMapNew2Old[new_id] = old_id

    ##------------------------------------------------------------------------
    def SetSpeciesTree( self, filename_tree ):
        """filename with species tree."""
        self.mFilenameSpeciesTree = filename_tree
    ##------------------------------------------------------------------------            
    def WriteTree( self, tree ):
        """write tree to file.
        """

        nexus = TreeTools.Newick2Nexus( tree )
        t = nexus.trees[0]
        TreeTools.MapTaxa( t, self.mMapOld2New )
        
        outfile = open( self.mTempdir + "/" + self.mFilenameTree, "w" )
        outfile.write("%i 1\n" % self.mNumSequences )
        outfile.write("%s\n" % TreeTools.Tree2Newick(t))
        outfile.close()

    ##------------------------------------------------------------------------        
    def Run( self, alignment, tree = None,
             dump = 0,
             test = False,
             options = {} ):

        self.mTempdir = tempfile.mkdtemp()
        self.mFilenameMali = "input"
        self.mFilenameOutput = "output"
        
        self.BuildMap( alignment )
        self.WriteAlignment( alignment )

        if test:
            print "# temporary directory is %s" % self.mTempdir
            
        if tree:
            self.mFilenameTree = "tree"
            
            ## check what kind of tree is given.
            if type(tree) == StringType:
                t = tree.strip()
                if not (t[0] == "(" and t[-1] in ");"):
                    tree = "".join(open( tree, "r").readlines())
                self.WriteTree( tree )

        extra_options = []

        if self.mFilenameSpeciesTree:
            extra_options.append( "-f %s" % os.path.abspath(self.mFilenameSpeciesTree ))
            
        statement = " ".join( ( self.mExecutable,
                                "best",
                                "-o %s" % self.mFilenameOutput,
                                " ".join(extra_options),
                                self.mFilenameMali) )

        s = subprocess.Popen( statement,
                              shell = True,
                              stdout = subprocess.PIPE,
                              stderr = subprocess.PIPE,
                              cwd = self.mTempdir,
                              close_fds = True)                              

        (out, err) = s.communicate()
        
        if s.returncode != 0:
            raise UsageError, "Error in running %s \n%s\n%s\nTemporary directory in %s" % (statement, err, out, self.mTempdir)

        if dump:
            self.mLog.write( "# stdout output of %s:\n%s\n######################################" % (self.mExecutable, out) )
            self.mLog.write( "# stderr output of %s:\n%s\n######################################" % (self.mExecutable, err) )

        lines = open( "%s/%s" % (self.mTempdir, self.mFilenameOutput), "r").readlines()

        if len(lines) == 0:
            raise UsageError, "Empty result from %s \n%s\n%s\nTemporary directory in %s" % (statement, err, out, self.mTempdir)            

        if dump:
            self.mLog.write( "# result output of %s:\n%s\n######################################\n" % (self.mExecutable, "".join(lines)) )

        if not test:
            shutil.rmtree( self.mTempdir )

        return self.parseOutput( lines, out, err )

    def parseOutput( self, lines, out, err ):

        lines = re.sub("\s", "", "".join(lines))
        lines = re.sub("\[[^\]]+\]", "", lines)
        
        t = TreeTools.Newick2Nexus( "".join(lines) )

        result = Result()
        t = t.trees[0]
        
        TreeTools.MapTaxa( t, self.mMapNew2Old )
        
        result.mTree = t

        result.mLog = out
        result.mErr = err
        
        return result

if __name__ == "__main__":
    
    parser = E.OptionParser( version = "%prog version: $Id: WrapperNJTree.py 2781 2009-09-10 11:33:14Z andreas $" )

    parser.add_option( "-m", "--map", dest="filename_map", type="string",
                       help="filename with mapping of species ids to swissprot species ids." )
    
    parser.add_option( "-t", "--tree", dest="filename_tree", type="string",
                       help="filename of trees." )

    parser.add_option("-a", "--filename-alignment", dest="filename_alignment", type="string",
                      help="filename with aligned codon sequences."  )

    parser.add_option( "--dump", dest="dump", action="store_true",
                      help="dump output."  )

    parser.set_defaults(
        separator="|",        
        dump = False,
        filename_map = None,
        filename_alignment = "-",
        filename_tree = None,
        )

    (options, args) = E.Start( parser )

    if options.filename_map:
        map_species2sp = IOTools.ReadMap( open(options.filename_map, "r"))

    E.debug( "species map: %s" % str(map_species2sp) )

    identifier_parser = IdentifierParserGPipe( map_species2sp = map_species2sp )

    njtree = NJTree( identifier_parser = identifier_parser )

    njtree.SetLog( options.stdlog )
    njtree.SetErr( options.stderr )    

    if options.filename_tree:
        njtree.SetSpeciesTree( options.filename_tree )
        
    mali = Mali.Mali()
    if options.filename_alignment == "-":
        infile = sys.stdin
    else:
        infile = open(options.filename_alignment, "r")
        
    mali.readFromFile( infile, format = "fasta")

    if mali.getLength() == 1:
        if options.loglevel >= 1:
            options.stdlog.write("# Warning: single gene tree\n" )
        options.stdout.write( "(%s:1);\n" % tuple(mali.getIdentifiers()) )
    elif mali.getLength() == 2:
        if options.loglevel >= 1:
            options.stdlog.write("# Warning: two gene tree\n" )
        options.stdout.write( "(%s:1,%s:1);\n" % tuple(mali.getIdentifiers()) )
    else:
        result = njtree.Run( mali,
                             dump = options.dump )
        
        options.stdout.write( str(result) + "\n" )

        if result.mLog:
            options.stdlog.write( str(result.mLog) + "\n" )

        if result.mErr:
            options.stderr.write( str(result.mErr) + "\n" )

    E.Stop()



################################################################################
#   Gene prediction pipeline 
#
#   $Id: tree2tree.py 2782 2009-09-10 11:40:29Z andreas $
#
#   Copyright (C) 2004 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
tree2tree.py - manipulate trees
===============================

:Author: Andreas Heger
:Release: $Id$

:Date: |today|
:Tags: Python

Purpose
-------

This script reads a collection of trees from stdin and outputs the again on stdout
after manipulating them. Manipulations include

   * renaming taxa

   * normalizing branch lengths

The complete list of methods is:

normalize
+++++++++

divide-by-tree
++++++++++++++

rename
++++++

set-uniform-branch-length
+++++++++++++++++++++++++

extract-with-pattern
++++++++++++++++++++

build-map
++++++++++++++++++++

remove-pattern
++++++++++++++++++++

unroot
++++++++++++++++++++

midpoint-root
++++++++++++++++++++

balanced-root
++++++++++++++++++++

add-node-names
++++++++++++++++++++

Usage
-----

Example::

   python <script_name>.py --help

Type::

   python <script_name>.py --help

for command line help.

Documentation
-------------

Code
----
"""

import os
import sys
import string
import re
import getopt
import time
import optparse
import math
import tempfile
import subprocess

import CGAT.Experiment as E
import CGAT.TreeTools as TreeTools
import CGAT.IOTools as IOTools
import CGAT.WrapperPhylip as WrapperPhylip

def Process( lines, other_trees, options, map_old2new, ntree):

    nexus = TreeTools.Newick2Nexus( map(lambda x: x[:-1], lines) )

    if options.loglevel >= 1:
        options.stdlog.write( "# read %i trees.\n" % len(nexus.trees))

    nskipped = 0
    ntotal = len(nexus.trees)
    extract_pattern = None
    species2remove = None
    write_map = False

    phylip_executable = None
    phylip_options = None

    index = 0

    ## default: do not output internal node names
    write_all_taxa = False
    
    for tree in nexus.trees:

        if options.outgroup:
            tree.root_with_outgroup( options.outgroup )

        for method in options.methods:

            if options.loglevel >=3:
                options.stdlog.write("# applying method %s to tree %i.\n" % (method, index))
                
            if method == "midpoint-root":
                tree.root_midpoint()

            elif method == "balanced-root":
                tree.root_balanced()

            elif method == "unroot":
                TreeTools.Unroot(tree)
                
            elif method=="phylip":
                if not phylip_executable:
                    phylip_executable=options.parameters[0]
                    del options.parameters[0]
                    phylip_options = re.split("@", options.parameters[0] )
                    del options.parameters[0]
                    
                    phylip = WrapperPhylip.Phylip()
                    phylip.setProgram( phylip_executable )
                    phylip.setOptions( phylip_options )

                phylip.setTree( tree )

                result = phylip.run()

                nexus.trees[index] = result.mNexus.trees[0]
                
            elif method == "normalize":
                if options.value == 0:
                    v = 0
                    for n in tree.chain.keys():
                        v = max( v, tree.node(n).data.branchlength )
                else:
                    v = options.value

                for n in tree.chain.keys():
                    tree.node(n).data.branchlength /= float( options.value )
                    
            elif method == "divide-by-tree":

                if len(other_trees) > 1:
                    other_tree = other_trees[ntree]
                else:
                    other_tree = other_trees[0]
                    
                ## the trees have to be exactly the same!!
                if options.loglevel >= 2:
                    print tree.display()
                    print other_tree.display()

                if not tree.is_identical( other_tree ):
                    nskipped += 1
                    continue

                ## even if the trees are the same (in topology), the node numbering might not be
                ## the same. Thus build a map of node ids.
                map_a2b = TreeTools.GetNodeMap( tree, other_tree )

                for n in tree.chain.keys():
                    try:
                        tree.node(n).data.branchlength /= float( other_tree.node(map_a2b[n]).data.branchlength )
                    except ZeroDivisionError:
                        options.stdlog.write("# Warning: branch for nodes %i and %i in tree-pair %i: divide by zero\n" %\
                                             ( n, map_a2b[n], ntree ))
                        continue

            elif method == "rename":
                if not map_old2new:
                    
                    map_old2new = IOTools.ReadMap( open(options.parameters[0], "r"), columns=(0,1) )

                    if options.invert_map:
                        map_old2new = IOTools.getInvertedDictionary( map_old2new, make_unique = True)
                        
                    del options.parameters[0]

                unknown = []
                for n, node in tree.chain.items():
                    if node.data.taxon:
                        try:
                            node.data.taxon = map_old2new[node.data.taxon]
                        except KeyError:
                            unknown.append( node.data.taxon )

                for taxon in unknown:
                    tree.prune( taxon )

            ## reformat terminals
            elif method == "extract-with-pattern":
                
                if not extract_pattern:
                    extract_pattern = re.compile(options.parameters[0])
                    del options.parameters[0]
                
                for n in tree.get_terminals():
                    node = tree.node( n )
                    node.data.taxon = extract_pattern.search( node.data.taxon ).groups()[0]

            elif method == "set-uniform-branchlength":
                for n in tree.chain.keys():
                    tree.node(n).data.branchlength = options.value

            elif method == "build-map":
                # build a map of identifiers
                options.write_map = True
                for n in tree.get_terminals():
                    node = tree.node( n )
                    if node.data.taxon not in map_old2new:
                        new = options.template_identifier % (len(map_old2new) + 1)
                        map_old2new[node.data.taxon] = new                        
                    node.data.taxon = map_old2new[node.data.taxon]

            elif method == "remove-pattern":
                if species2remove == None:
                    species2remove = re.compile(options.parameters[0])
                    del options.parameters
                taxa = []
                for n in tree.get_terminals():
                    t = tree.node(n).data.taxon
                    skip = False
                    if species2remove.search( t ):
                        continue
                    if not skip:
                        taxa.append(t)
                TreeTools.PruneTree( tree, taxa )

            elif method == "add-node-names":

                inode = 0
                write_all_taxa = True
                for n, node in tree.chain.items():
                    if not node.data.taxon:
                        node.data.taxon = "inode%i" % inode
                        inode += 1

            elif method == "newick2nhx":
                ## convert names to species names
                for n in tree.get_terminals():
                    t = tree.node(n).data.taxon
                    d = t.split("|")
                    if len(d) >= 2:
                        tree.node(n).data.species = d[0]
                        
        index += 1
        ntree += 1

    if options.output_format == "nh":
        options.stdout.write(TreeTools.Nexus2Newick( nexus, write_all_taxa = True,
                                                     with_branchlengths = options.with_branchlengths) + "\n" )
    else:
        for tree in nexus.trees:
            tree.writeToFile( options.stdout, format = options.output_format )

    return ntotal, nskipped, ntree

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: tree2tree.py 2782 2009-09-10 11:40:29Z andreas $",
                                    usage = globals()["__doc__"] )

    parser.add_option("-d", "--value", dest="value", type="float",
                      help="normalizing value."  )
    parser.add_option("-m", "--method", dest="methods", type="string",
                      help="""methods to apply [normalize|divide-by-tree|divide-by-tree|rename|set-uniform-branch-length|extract-with-pattern|build-map|remove-pattern|unroot|midpoint-root|balanced-root|add-node-names"""  )
    parser.add_option("-2", "--filename-tree2", dest="filename_tree2", type="string",
                      help="filename with second tree."  )
    parser.add_option("-o", "--outgroup", dest="outgroup", type="string",
                      help="reroot with outgroup before processing.")
    parser.add_option("-p", "--parameters", dest="parameters", type="string",
                      help="parameters for methods.")
    parser.add_option("-e", "--template-identifier", dest="template_identifier", type="string",
                      help="""template identifier [%default]. A %i is replaced by the position
                      of the sequence in the file."""  )
    parser.add_option("-i", "--invert-map", dest="invert_map", action="store_true",
                      help="""invert map.""")
    parser.add_option("-f", "--filter", dest="filter", type="choice",
                      choices=("max-branch-length",),
                      help="filter trees")
    parser.add_option("--output-format", dest="output_format", type="choice",
                      choices=( "nh", "nhx" ),
                      help=("output format for trees."))
    parser.add_option("-b", "--no-branch-lengths", dest="with_branchlengths", action="store_false",
                      help="""do not write branchlengths. Per default, 0 branch lengths are added.""")

    parser.set_defaults(
        value = 0,
        methods = "",
        filename_tree2 = None,
        outgroup = None,
        parameters = "",
        template_identifier="ID%06i",
        write_map = False,
        invert_map = False,
        filter = None,
        output_format = "nh",
        with_branchlengths = True,
        )

    (options, args) = E.Start( parser, add_pipe_options = True )

    options.methods = options.methods.split(",")
    options.parameters = options.parameters.split(",")    

    other_trees = []
    ## read other trees
    if options.filename_tree2:
        other_nexus = TreeTools.Newick2Nexus( open(options.filename_tree2, "r") )
        if len(other_nexus.trees) > 0:
            other_trees = other_nexus.trees
        else:
            other_tree = other_nexus.trees[0]
            other_trees = [other_tree]

    lines = sys.stdin.readlines()

    ntotal, nskipped, ntree = 0, 0, 0

    if options.filter:

        nexus = TreeTools.Newick2Nexus( lines )
        
        new_trees = []

        value = float(options.parameters[0])
        del options.parameters[0]

        ## decision functions: return true, if tree
        ## is to be skipped
        if options.filter == "max-branch-length":
            f = lambda x: x >= value

        for tree in nexus.trees:
            ntotal += 1

            for id, node in tree.chain.items():
                if f(node.data.branchlength):
                    nskipped += 1
                    break
            else:
                new_trees.append(tree)
                ntree += 1
                
        nexus.trees = new_trees

        options.stdout.write(TreeTools.Nexus2Newick( nexus, with_names = True ) + "\n" )
        
    else:

        ## iterate over chunks
        chunks = filter( lambda x: lines[x][0] == ">", range(len(lines)))

        map_old2new = {}

        if chunks:
            for c in range(len(chunks)-1):
                a, b = chunks[c], chunks[c+1]
                options.stdout.write( lines[a] )
                a += 1
                Process( lines[a:b], other_trees, options, map_old2new,ntree )

            options.stdout.write( lines[chunks[-1]] )
            t, s, ntree = Process( lines[chunks[-1]+1:], other_trees, options, map_old2new, ntree )
            ntotal += t
            nskipped += s
        else:
            ntotal, nskipped, ntree = Process( lines, other_trees, options, map_old2new,ntree )

        if options.write_map:
            p = options.parameters[0]
            if p:
                outfile = open(p, "w")
            else:
                outfile = options.stdout

            outfile.write("old\tnew\n")            
            for old_id, new_id in map_old2new.items():
                outfile.write("%s\t%s\n" % (old_id, new_id) )
            if p:
                outfile.close()

    if options.loglevel >= 1:
        options.stdlog.write( "# ntotal=%i, nskipped=%i\n" % (ntotal, nskipped))
        
    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
codonbias_acai2tsv.py - codonbias analysis
=========================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python codonbias_acai2tsv.py --help

Type::

   python codonbias_acai2tsv.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import tempfile
import subprocess
import optparse
import random

"""Wrapper for adaptive codon bias program
"""

import CGAT.Experiment as E
import CGAT.Genomics as Genomics

import CGAT.WrapperAdaptiveCAI as WrapperAdaptiveCAI
import CGAT.IOTools as IOTools
import CGAT.CSV as CSV

## order of codon matrix as expected by caijava.
OUTPUT_ORDER_CODON_MATRIX = ( \
    ("TTT","TTC","TTA","TTG","TCT","TCC","TCA","TCG"),
    ("TAT","TAC","TAA","TAG","TGT","TGC","TGA","TGG"),
    ("CTT","CTC","CTA","CTG","CCT","CCC","CCA","CCG"),
    ("CAT","CAC","CAA","CAG","CGT","CGC","CGA","CGG"),
    ("ATT","ATC","ATA","ATG","ACT","ACC","ACA","ACG"),
    ("AAT","AAC","AAA","AAG","AGT","AGC","AGA","AGG"),
    ("GTT","GTC","GTA","GTG","GCT","GCC","GCA","GCG"),
    ("GAT","GAC","GAA","GAG","GGT","GGC","GGA","GGG"),
)

# a list of codon preferences (counts)
CODON_PREFERENCES = {
    # taken from Shields et al. (1988), table 2
    'dmelanogaster' : {
    "TTT" : 13,
    "TTC" : 145,
    "TTA" : 3,
    "TTG" : 39,
    "TCT" : 30,
    "TCC" : 127,
    "TCA" : 5,
    "TCG" : 57,
    "TAT" : 32,
    "TAC" : 140,
    "TAA" : 14, # stop
    "TAG" : 1, # stop
    "TGT" : 3,
    "TGC" : 54,
    "TGA" : 0, # stop
    "TGG" : 39,
    "CTT" : 14,
    "CTC" : 43,
    "CTA" : 7,
    "CTG" : 236,
    "CCT" : 21,
    "CCC" : 138,
    "CCA" : 31,
    "CCG" : 14,
    "CAT" : 13,
    "CAC" : 72,
    "CAA" : 12,
    "CAG" : 156,
    "CGT" : 75,
    "CGC" : 97,
    "CGA" : 4,
    "CGG" : 0,
    "ATT" : 61,
    "ATC" : 213,
    "ATA" : 0,
    "ATG" : 114,
    "ACT" : 34,
    "ACC" : 216,
    "ACA" : 5,
    "ACG" : 12,
    "AAT" : 17,
    "AAC" : 185,
    "AAA" : 13,
    "AAG" : 346,
    "AGT" : 1,
    "AGC" : 63,
    "AGA" : 1,
    "AGG" : 9,
    "GTT" : 56,
    "GTC" : 131,
    "GTA" : 9,
    "GTG" : 163,
    "GCT" : 95,
    "GCC" : 299,
    "GCA" : 18,
    "GCG" : 19,
    "GAT" : 126,
    "GAC" : 160,
    "GAA" : 31,
    "GAG" : 313,
    "GGT" : 130,
    "GGC" : 160,
    "GGA" : 83,
    "GGG" : 0,
    }
    }


if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: codonbias_acai2tsv.py 865 2007-01-15 13:44:43Z andreas $")

    parser.add_option("-o", "--input-file-trace", dest="input_filename_trace", type="string",
                      help="input filename for cai.",
                      metavar="FILE" )

    parser.add_option("-e", "--input-file-genes", dest="input_filename_genes", type="string",
                      help="input filename for genes information from cai.",
                      metavar="FILE" )

    parser.add_option("-c", "--input-file-codons", dest="input_filename_codons", type="string",
                      help="input filename for codon usage information.",
                      metavar="FILE" )

    parser.add_option( "--input-file-sequences", dest="input_filename_sequences", type="string",
                      help="input filename with sequences.",
                      metavar="FILE" )

    parser.add_option("-t", "--input-file-subset", dest="input_filename_subset", type="string",
                      help="input filename with subset.",
                      metavar="FILE" )

    parser.add_option("--codon-table-format", dest="codon_table_format", type="choice",
                      choices=("list", "matrix"),
                      help="output options for output codon tables." )

    parser.add_option("--codon-table-type", dest="codon_table_type", type="choice",
                      choices=("counts", "frequencies", "weights", "absolute-frequencies"),
                      help="type of codon table." )

    parser.add_option("-r", "--reference", dest="reference", type="string",
                      help="dump CAI reference weights for species." )

    parser.add_option("-s", "--select", dest="select", type="string",
                      help="fields to select from genes table." )

    parser.add_option("-m", "--map", dest="input_filename_map", type="string",
                      help="filename with mapping information for gene names.",
                      metavar="FILE")

    parser.add_option("-i", "--invert-map", dest="invert_map", action="store_true",
                      help="invert map.")

    parser.add_option("-d", "--dominant-set", dest="dominant_set", type ="float",
                      help="only print out dominant set (# fraction of most biased genes).")

    parser.add_option("--reverse-set", dest="reverse_set", action="store_true",
                      help="print the reverse set, i.e., then non-dominant set.")

    parser.add_option("-u", "--codon-usage", dest="codon_usage", type="string",
                      help="print codon usage for the full/biased set of genes [full|biased].")

    parser.add_option("-w", "--weights", dest="weights", type="string",
                      help="print weights [final-list|final-matrix|random|compute|weights|frequencies|absolute-frequencies].")

    parser.add_option( "--weights-matrix2table", dest="weights_matrix2table", action="store_true",
                      help="convert a weights matrix to a weights table.")

    parser.add_option( "--get-preferred-codons", dest="get_preferred_codons", type="string",
                      help="compute overview of preferred codons.")

    parser.set_defaults( \
        input_filename = "-",
        input_filename_trace = None,
        input_filename_genes = None,
        input_filename_codons = None,
        input_filename_map = None,
        input_filename_subset = None,
        input_filename_sequences = None,
        invert_map = False,
        select = None,
        codon_usage = None,
        weights = None,
        revserse_set = False,
        pseudocounts = 1,
        codon_table_format = "list",
        codon_table_type = "weights",
        weights_matrix2table = False,
        random_size = 1000,
        get_preferred_codons = None,
        dominant_set = 0.0 )

    (options, args) = E.Start( parser )
    if options.select:
        options.select = options.select.split(",")

    outfile = options.stdout

    ###################################################################
    ## convert weights table to a codon table
    if options.weights_matrix2table:
        lines = options.stdin.readlines()
        data = []
        for line in lines:
            if line[0] == "#": continue
            data += list(map(float, line[:-1].split(",")))

        weights = {}
        x = 0
        for cc in OUTPUT_ORDER_CODON_MATRIX:
            for c in cc:
                weights[c] = data[x]
                x += 1

        outfile.write ("CODON\tWEIGHT\n")
        codons = weights.keys()
        codons.sort()
        for codon in codons:
            outfile.write( "%s\t%f\n" % ( codon, weights[codon]) )

        E.Stop()
        sys.exit(1)

    ###################################################################
    map_genes = {}

    if options.input_filename_map:
        data = map( lambda x: x[:-1].split("\t")[:2], filter( lambda x: x[0] != "#", open( options.input_filename_map, "r").readlines()))

        for a, b in data:
            if options.invert_map: a,b = b,a
            map_genes[a] = b

    result = WrapperAdaptiveCAI.AdaptiveCAIResult()

    if options.input_filename_genes:
        gene_file = open(options.input_filename_genes,"r")
    else:
        gene_file = None

    if options.input_filename_codons:
        codon_file = open(options.input_filename_codons,"r")
    else:
        codon_file = None

    if options.input_filename_trace:
        trace_file = open(options.input_filename_trace,"r")
    else:
        trace_file = None

    if options.input_filename_subset:
        l, e = IOTools.ReadList( open(options.input_filename_subset,"r") )
        subset = set(l)
        if options.loglevel >= 1:
            options.stdlog.write( "# read %i entries into subset from %s.\n" % (len(subset), options.input_filename_subset))
    else:
        subset = None
        
    result.Read( gene_file=gene_file, codon_file=codon_file, trace_file = trace_file )

    if gene_file: gene_file.close()
    if codon_file: codon_file.close()
    if trace_file: trace_file.close()    

    if options.reference:
        if options.reference not in CODON_PREFERENCES:
            raise "unknown species %s: possibles species are: %s" % (options.reference, str(CODON_PREFERNCES.keys()))

        weights = Genomics.CalculateCAIWeightsFromCounts( CODON_PREFERENCES[options.reference], options.pseudocounts )
        
        for x in range(len(OUTPUT_ORDER_CODON_MATRIX)):
            outfile.write(",".join( map(lambda z: "%5.3f" % z, [ weights[codon.upper()] for codon in OUTPUT_ORDER_CODON_MATRIX[x]])))
            outfile.write("\n")

    if options.dominant_set and gene_file:
        cai_threshold = result.GetDominantThreshold( options.dominant_set )
    else:
        if options.reverse_set:
            cai_threshold = 1.0
        else:
            cai_threshold = 0.0
        
    if options.select:

        fields = []
        titles = []
        for x in options.select:
            f = re.match("(\S+) (AS|as) (\S+)", x)
            if f:
                fields.append(f.groups()[0].upper())
                titles.append(f.groups()[2])
            else:
                fields.append(x.upper())
                titles.append(x)
                
        outfile.write( "GENENAME\t" + string.join(titles, "\t") + "\n")
        
        for genename,data in result.mGeneInfo.items():
            if genename in map_genes: genename = map_genes[genename]

            if options.reverse_set:
                if data["CAICLASS"] >= cai_threshold: continue                
            else:
                if data["CAICLASS"] < cai_threshold: continue
            
            outfile.write( genename )
            for c in fields:
                outfile.write("\t%s" % str(data[c]))
            outfile.write("\n")

    if options.weights:

        format = options.codon_table_format

        if options.weights in ("compute-counts", "compute-weights", "compute-frequencies"):
            ## compute codon usage weights from a set of sequences
            codons = CODON_PREFERENCES["dmelanogaster"].keys()
            counts = {}
            for x in codons: counts[x] = 0
            
            if options.input_filename_sequences:
                sequences = Genomics.ReadPeptideSequences( open( options.input_filename_sequences, "r"), filter = subset )
                for key, sequence in sequences.items():
                    sequence = re.sub(" ", "", sequence )
                    if len(sequence) % 3 != 0:
                        raise "warning: sequence %s is not multiple of 3" % key
                    for codon in [ sequence[x:x+3] for x in range(0, len(sequence), 3)]:
                        counts[codon.upper()] += 1
                        
            if options.weights == "compute-frequencies":
                weights = Genomics.CalculateCodonFrequenciesFromCounts( counts, options.pseudocounts )
            elif options.weights == "compute-weights":
                weights = Genomics.CalculateCAIWeightsFromCounts( counts, options.pseudocounts )
            else:
                weights = counts
            
        elif options.weights in ("final-list", "final-matrix"):
            
            weights = result.mFinalWeights
            if options.weights == "final-list":
                format = "list"
            else:
                format = "matrix"

        elif options.weights == "random":
            ## get random weights
            codons = CODON_PREFERENCES["dmelanogaster"].keys()
            counts = {}
            for x in codons:
                counts[x] = random.randint( 1, options.random_size )
                
            weights = Genomics.CalculateCAIWeightsFromCounts( counts, options.pseudocounts )
            format = "matrix"

        elif options.weights == "biased":
            ## get biased weights
            codons = Genomics.GetUniformCodonUsage()

            weights = Genomics.CalculateCAIWeightsFromCounts( counts, options.pseudocounts )
            format = "matrix"

        elif options.weights in ( "uniform-weights", "uniform-frequencies"):
            ## get uniform weights
            codons = Genomics.GetUniformCodonUsage()

            if options.weights == ( "uniform-weights" ):
                weights = Genomics.CalculateCAIWeightsFromCounts( counts, options.pseudocounts )
                format = "matrix"
            else:
                weights = codons
                format = "list"

        elif options.weights in ("counts", "frequencies", "absolute-frequencies"):
            ## get weights as frequencies
            ## compute from scratch. In the caijava file, the absolute frequencey f / gene_length is
            ## given. Thus the total number of codons is f * gene_length.
            codons = CODON_PREFERENCES["dmelanogaster"].keys()            
            counts = {}
            for c in codons: counts[c] = 0
            
            for genename, data in result.mGeneInfo.items():
                
                if options.reverse_set:
                    if data["CAICLASS"] >= cai_threshold: continue                
                else:
                    if data["CAICLASS"] < cai_threshold: continue
                    
                l = data["GENELENGTH"]
                for c in codons:
                    counts[c] += int(data[c] * l)

            if options.weights == "frequencies":
                weights = Genomics.CalculateCodonFrequenciesFromCounts( counts, options.pseudocounts )
            elif options.weights == "counts":
                weights = counts
            elif options.weights == "absolute-frequencies":
                ## compute absolute frequencies (with pseudo-counts, but do not normalize per aa)
                weights = {}
                m = sum(counts.values())
                for k,v in counts.items():
                    weights[k] = float(v) / m
                
            format = "list"

        elif options.weights == "subset":

            codons = CODON_PREFERENCES["dmelanogaster"].keys()            
            counts = {}
            for c in codons: counts[c] = 0
            
            for genename, data in result.mGeneInfo.items():

                found = genename in subset
                if (not found and not options.reverse_set) or (found and options.reverse_set):
                    continue
                    
                l = data["GENELENGTH"]
                for c in codons:
                    counts[c] += int(data[c] * l)
                    
            if options.codon_table_type == "frequencies":
                weights = Genomics.CalculateCodonFrequenciesFromCounts( counts, options.pseudocounts )                
            elif options.codon_table_type == "weights":                
                weights = Genomics.CalculateCAIWeightsFromCounts( counts, options.pseudocounts )
            elif options.codon_table_type == "counts":
                weights = counts
            if options.codon_table_type == "absolute-frequencies":
                ## compute absolute frequencies (with pseudo-counts, but do not normalize per aa)
                weights = {}
                m = sum(counts.values())
                for k,v in counts.items():
                    weights[k] = float(v) / m
                
        else:
            raise "unknown weights %s" % options.weights
        
        if format == "list":
            outfile.write ("CODON\tWEIGHT\n")
            codons = weights.keys()
            codons.sort()
            for codon in codons:
                outfile.write( "%s\t%f\n" % ( codon, weights[codon]) )
                
        elif format == "matrix":

            for x in range(len(OUTPUT_ORDER_CODON_MATRIX)):
                outfile.write(",".join( map(lambda z: "%5.3f" % z, [ weights[codon.upper()] for codon in OUTPUT_ORDER_CODON_MATRIX[x]])))
                outfile.write("\n")
        
    if options.codon_usage:
        outfile.write( "CODON\tFREQUENCY\n" )

        if options.codon_usage == "biased":
            usages = result.mCodonUsages[-1]
        elif options.codon_usage == "full":
            usages = result.mCodonUsages[0]
        elif options.codon_usage == "weights":
            usages = WrapperAdaptiveCAI.CalculateWeightsFromUsage(result.mCodonUsages[0])
        else:
            raise "unknown option '%s' for codon-usage." % options.codon_usage

        codons = usages.keys()
        codons.sort()
        for codon in codons:
            outfile.write( "%s\t%f\n" % ( codon, usages[codon]) )

    E.Stop()
    



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
WrapperClustal.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import os, sys, string, re, tempfile

from Bio.Clustalw import MultipleAlignCL
from Bio.Clustalw import do_alignment

"""Wrapper for Clustal based on Biopython
"""

import Genomics

class Clustal:

    mOptions = ""
    
    def __init__( self, options = ""):
        self.mOptions = options

    def Align( self, s1, s2, result ):

        result.clear()

        handle_tmpfile1, filename_tmpfile1 = tempfile.mkstemp()
        handle_tmpfile2, filename_tmpfile2 = tempfile.mkstemp()        
        os.write( handle_tmpfile1, ">s1\n%s\n" % (s1))
        os.write( handle_tmpfile1, ">s2\n%s\n" % (s2))
        os.close( handle_tmpfile1 )
        os.close( handle_tmpfile2 )
        cline = MultipleAlignCL( filename_tmpfile1 )
        cline.set_output( filename_tmpfile2 )
                  
        align = do_alignment(cline)
                  
        seqs = align.get_all_seqs()
        if len(seqs) != 2: return result
        
        a1 = seqs[0].seq.tostring()
        a2 = seqs[1].seq.tostring()

        x1 = 0
        x2 = 0
        for pos in range(len(a1)):
            
            if a1[pos] not in "Nn-" and a2[pos] not in "Nn-":
                result.addPair( x1, x2, 0 )
                x1 += 1
                x2 += 1
                continue
            
            if a1[pos] != "-": x1 += 1
            if a2[pos] != "-": x2 += 1            

            
        os.remove( filename_tmpfile1 )
        os.remove( filename_tmpfile2 )
            
        return result




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2781 2009-09-10 11:33:14Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
cat_tables.py - concatenate tables
==================================

:Author: Andreas Heger
:Release: $Id: cgat_script_template.py 2781 2009-09-10 11:33:14Z andreas $
:Date: |today|
:Tags: Python

Purpose
-------

concatenate tables. Headers of subsequent files are ignored.

Usage
-----

Type::

   python <script_name>.py --help

for command line help.

Code
----

""" 

import os
import sys
import re
import optparse
import fileinput

import CGAT.Experiment as E

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2781 2009-09-10 11:33:14Z andreas $", usage = globals()["__doc__"] )

    parser.set_defaults(
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    if len(args) == 0 or (len(args) == 1 and args[0] == "-"):
        infile = options.stdin
    else:
        infile = fileinput.FileInput( args )
        
    ## do sth
    ninput, nskipped, noutput = 0, 0, 0

    header = False

    for line in infile:
        ninput += 1
        if line.startswith( "#" ):
            pass
        elif not header:
            header = line
        elif line == header:
            nskipped += 1
            continue
            
        options.stdout.write( line )
        noutput += 1

    E.info( "ninput=%i, noutput=%i, nskipped=%i" % (ninput, noutput,nskipped) )

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )




"""
Histogram2D.py - functions for handling two-dimensional histograms.
===================================================================

:Author: 
:Release: $Id$
:Date: |today|
:Tags: Python

"""

import string

def Calculate(values, mode=0, bin_function=None):
    """Return a list of (value, count) pairs, summarizing the input values.
    Sorted by increasing value, or if mode=1, by decreasing count.

    If bin_function is given, map it over values first.
    
    """
    
    if bin_function: values = map(bin_function, values)
    
    bins = {}
    for val in values:
        v = "%f-%f" % tuple(val)
        bins[v] = bins.get(v, 0) + 1

    bb = bins.items()

    
    if mode:
        bb.sort(lambda x,y: cmp(y[1],x[1]))
    else:
        bb.sort()

    r = []
    for v,n in bb:
        x,y = map(string.atof, string.split(v, "-"))
        r.append( (x,y,n) )
        
    return r

#-------------------------------------------------------------------------------------------------------
def Print( h, bin_function=None ):
    """print a histogram.

    A histogram can either be a list/tuple of values or
    a list/tuple of lists/tuples where the first value contains
    the bin and second contains the values (which can again be
    a list/tuple).

    :param format: output format. 
        0 = print histogram in several lines,
        1 = print histogram on single line
    """

    if bin_function: h = map(bin_function, h)

    for hh in h:
        print string.join( map(str, hh), "\t")
        






from pysam.csamtools cimport *

import collections, array, struct, itertools
import CGAT.Experiment as E

cdef class SetNH:

    cdef iter
    cdef stack

    def __cinit__(self, iter ):
        self.iter = itertools.groupby( iter, lambda x: x.qname )
        self.stack = []

    def __iter__(self):
        return self

    def __next__(self): 
        """python version of next().
        """
        
        while 1:
            if self.stack:
                return self.stack.pop(0)
            else:
                key, x = self.iter.next()
                self.stack = list(x)
                nh = len(self.stack)
                for read in self.stack:
                    if not read.is_unmapped:
                        t = dict( read.tags )
                        t['NH'] = nh
                        read.tags = list(t.iteritems())

def filter_bam( Samfile input_samfile,
                Samfile output_samfile,
                Samfile reference_samfile,
                remove_nonunique = False,
                remove_contigs = None,
                remove_unmapped = False,
                remove_mismatches = False,
                colour_mismatches = False ):

    '''
    To conserve memory, the tid and NM flag from *transcripts_samfile*
    are packed into memory. As a consequence, this method requires 
    max(NM) < 256 (2^8) and max(tid) < 16777216 (2^24)

    If *remove_nonunique* is set, only uniquely matching reads will be output.

    If *remove_contigs* is given, contigs that are in remove_contigs will
    be removed. Note that this will only remove the alignment, but not
    all alignments of a particuluar read and the NH flag will *NOT* be
    updated.

    If *remove_unmapped* is given, unmapped reads will be removed.

    If *remove_mismatches* is set, only reads with number of mismatches
    better than in reference_samfile will be kept.

    If *colour_mismatches* is set, the ``CM`` tag will be used
    to count differences. By default, the ``NM`` tag is used.
    The tag that is used needs to present in both *input_samfile*
    and *reference_samfile*.
    '''

    cdef int ninput = 0
    cdef int nunmapped = 0
    cdef int nmismatches = 0
    cdef int noutput = 0
    cdef int nnonunique = 0
    cdef int nremoved_contigs = 0

    cdef bint c_remove_nonunique = remove_nonunique
    cdef bint c_remove_mismatches = remove_mismatches
    cdef bint c_remove_unmapped = remove_unmapped

    cdef int * remove_contig_tids 
    cdef int nremove_contig_tids = 0
    cdef AlignedRead read
    cdef AlignedRead match

    E.info( "building index" )
    # build index
    # this method will start indexing from the current file position
    # if you decide
    cdef int ret = 1#
    cdef int x
    cdef bam1_t * b = <bam1_t*>calloc(1, sizeof( bam1_t) )
    cdef uint64_t pos
    cdef uint8_t * v
    cdef int32_t nm
    cdef int32_t nh
    cdef int tid
    cdef int transript_tid
    cdef long val
    cdef bint skip = 0

    cdef int32_t read_mismatches = 0

    # decide which tag to use
    cdef char * nm_tag = 'NM' 
    cdef char * cm_tag = 'CM'
    cdef char * tag

    if colour_mismatches:
        tag = cm_tag
    else:
        tag = nm_tag

    if c_remove_mismatches:
        if not reference_samfile:
            raise ValueError("require another bam file for mismatch filtering" )

        # L = 1 byte (unsigned char)
        def _gen(): return array.array('B') 
        index = collections.defaultdict( _gen )

        while ret > 0:
            ret = samread( reference_samfile.samfile, b)
            if ret > 0:
                # ignore unmapped reads
                if b.core.flag & 4: continue
                qname = bam1_qname( b )
                tid = b.core.tid
                v = bam_aux_get(b, tag)
                nm = <int32_t>bam_aux2i(v)
                index[qname].append( nm )

    # setup list of contigs to remove:
    if remove_contigs:
        nremove_contig_tids = len(remove_contigs)
        remove_contig_tids = <int*>malloc( sizeof(int) * nremove_contig_tids )
        for x, rname in enumerate( remove_contigs):
            remove_contig_tids[x] = input_samfile.gettid( rname )

    bam_destroy1( b )
    
    E.info( "built index for %i reads" % len(index))
    
    E.info( "starting filtering" )

    for read in input_samfile:

        ninput += 1
        # if ninput > 10000: break

        # remove unmapped reads
        if read._delegate.core.flag & 4:
            if c_remove_unmapped:
                nunmapped += 1
            else:
                output_samfile.write( read )
            continue

        # remove non-unique alignments
        if c_remove_nonunique:
            v = bam_aux_get(read._delegate, 'NH')
            if v != NULL:
                nh = <int32_t>bam_aux2i(v)
                if nh > 1: 
                    nnonunique += 1
                    continue

        # remove reads matched to certain contigs
        if nremove_contig_tids:
            skip = 0
            for x from 0 <= x < nremove_contig_tids:
                if remove_contig_tids[x] == read.tid:
                    skip = 1
                    break
            if skip: 
                nremoved_contigs += 1
                continue

        # remove reads in other bam file if their are better matching
        if c_remove_mismatches:
            if read.qname in index:
                # can compress index before, depends on
                # how many reads you expect test to filter
                nm = min(index[read.qname])
                read_mismatches = read.opt(tag)
                if nm > read_mismatches: 
                    nmismatches += 1
                    continue
                
        noutput += 1
        output_samfile.write( read )

    c = E.Counter()
    c.input = ninput
    c.removed_nonunique = nnonunique
    c.removed_contigs = nremoved_contigs
    c.output = noutput
    c.removed_unmapped = nunmapped
    c.removed_mismatches = nmismatches

    if nremove_contig_tids:
        free( remove_contig_tids )

    E.info( "filtering finished" )

    return c



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
numbers2rgb.py - map numbers to RGB values
==========================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

Reads numbers from stdin and maps to random RGB values.

Usage
-----

Example::

   python numbers2rgb.py --help

Type::

   python numbers2rgb.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import optparse
import time
import random

import CGAT.Experiment as E

if __name__  == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: numbers2rgb.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.add_option( "-m", "--method", dest="method", type="string" ,
                       help="method to use.")

    parser.set_defaults(
        method = "random_rgb",
        )
    
    (options, args) = E.Start( parser, add_pipe_options = True )

    numbers = map( lambda x: int(x[:-1].split("\t")[0]),
                   filter( lambda x: x[0] != "#", sys.stdin.readlines()) )

    if options.method == "random_rgb":
        f = lambda x: "%i,%i,%i" % (random.randint(0,256),
                                    random.randint(0,256),
                                    random.randint(0,256) )

    map_number2output = {}
    for x in numbers:
        if x not in map_number2output:
            map_number2output[x] = f(x)

    for x, y in map_number2output.items():
        options.stdout.write( "%i\t%s\n" % (x, y) )

    E.Stop()
    



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
graph_cluster_by_species.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python graph_cluster_by_species.py --help

Type::

   python graph_cluster_by_species.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt
import tempfile
import time
import popen2
import optparse

USAGE="""python %s [OPTIONS] < graph.in > graph.out

Version: $Id: graph_cluster_by_species.py 2782 2009-09-10 11:40:29Z andreas $

Cluster transcripts based on a distance measure. The transcripts can be clustered by

   * species: Find best matching clusters including as many species as possible. Used to
        select a subset of 1:1 orthologous genes. Better though is to use a tree.
        
   * genes: build clusters out of best matching transcripts. Used to build clusters of orthologous
        transcripts from a cluster of orthologs. Clustering is done by reciprocal best hit.
        Identical transcripts have a distance of 0 and are grouped together.
        
""" % sys.argv[0]

import CGAT.Experiment as E
import networkx

###########################################################
def AddSynonym( rep_transcript, mem_transcript, map_synonyms, map_to_synonyms):

    if rep_transcript not in map_synonyms:
        map_synonyms[rep_transcript] = []

    if mem_transcript in map_synonyms:            

        for s in map_synonyms[mem_transcript]:
            map_to_synonyms[s] = rep_transcript

        map_synonyms[rep_transcript] += map_synonyms[mem_transcript]                    
        del map_synonyms[mem_transcript]

    map_synonyms[rep_transcript].append( mem_transcript )
    map_to_synonyms[mem_transcript] = rep_transcript

###########################################################
def clusterBySpecies( links, options ):
    """cluster links in infile by species."""
    
    map_priority = {}
    for x in range( len(options.quality_priority) ):
        map_priority[options.quality_priority[x]] = x
        
    transcripts = {}
    genomes = {}
    graph = {}
    reciprocal_best = {}
    map_synonyms = {}
    map_to_synonyms = {}
    

    ## build synonym list. Use priority list
    for transcript1, transcript2, weight in links:
    
        weight = float(weight)

        species1, t1, gene1, quality1 = transcript1.split(options.separator)
        species2, t2, gene2, quality2 = transcript2.split(options.separator)

        if weight <= options.synonym_weight and species1 == species2:

            ## use priority to decide about synonym
            if map_priority[quality1] > map_priority[quality2]:
                transcript1, transcript2 = transcript2, transcript1

            AddSynonym( transcript1, transcript2, map_synonyms, map_to_synonyms )
            
    ## second parse for actual processing. Synonyms are mapped
    ninput, noutput, nskipped = 0, 0, 0
    
    for transcript1, transcript2, weight in links:

        weight = float(weight)
        ninput += 1
        
        if options.min_weight != None and weight < options.min_weight:
            nskipped += 1
            continue

        if options.max_weight != None and weight > options.max_weight:
            nskipped += 1
            continue

        if transcript1 in map_to_synonyms: transcript1 = map_to_synonyms[transcript1]
        if transcript2 in map_to_synonyms: transcript2 = map_to_synonyms[transcript2]        

        if transcript1 > transcript2: transcript1, transcript2 = transcript2, transcript1
        
        species1 = transcript1.split(options.separator)[0]
        species2 = transcript2.split(options.separator)[0]

        if species1 not in transcripts: transcripts[species1] = {}
        if species2 not in transcripts: transcripts[species2] = {}
        
        transcripts[species1][transcript1] = 1
        transcripts[species2][transcript2] = 1        

        genomes[species1] = 1
        genomes[species2] = 1

        key = "%s-%s" % (species1, species2)
        if key not in graph: graph[key] = []
        graph[key].append( (weight, transcript1, transcript2) )
        
        key = "%s-%s" % (species2, species1)
        if key not in graph: graph[key] = []
        graph[key].append( (weight, transcript1, transcript2) )

        ## compute reciprocal best matches
        if transcript1 not in reciprocal_best: reciprocal_best[transcript1] = {}
        if species2 not in reciprocal_best[transcript1]:
            reciprocal_best[transcript1][species2] = (weight, transcript2)
        else:
            if weight < reciprocal_best[transcript1][species2][0]:
                reciprocal_best[transcript1][species2] = (weight, transcript2)

        if transcript2 not in reciprocal_best: reciprocal_best[transcript2] = {}
        if species1 not in reciprocal_best[transcript2]:
            reciprocal_best[transcript2][species1] = (weight, transcript1)
        else:
            if weight < reciprocal_best[transcript2][species1][0]:
                reciprocal_best[transcript2][species1] = (weight, transcript1)

        noutput += 1
        
    if options.loglevel >= 1:
        options.stdlog.write( "# ninput=%i, noutput=%i, nskipped=%i, ngenomes=%i\n" % (ninput, noutput, nskipped, len(genomes) ))

    ## Compute reciprocal best match graph
    subgraph = networkx.Graph()        
    for k, v in reciprocal_best.items():
        subgraph.add_node( k )
        
        this_genome = k.split(options.separator)[0]
        
        for other_genome, best in v.items():
            
            if this_genome == other_genome: continue
            
            best_weight, best_transcript = best
            if reciprocal_best[best_transcript][this_genome][1] == k:

                subgraph.add_edge( k, best_transcript )

    ## compute components
    components = networkx.connected_components( subgraph )

    if options.loglevel >= 1:
        options.stdlog.write( "# component sizes before pruning: %s\n" % ",".join(map( lambda x: str(len(x)), components)))

    ## prune components: eliminate sequences from the same species in each component
    new_components = []

    ## sort by species and quality, take first entry, the rest are synonyms
    for component in components:
        component = map( lambda x: (x.split(options.separator)[0], x.split(options.separator)[1], x), component)
        component.sort()
        new_component = []
        last_species = None
        rep_transcript = None
        for species, quality, transcript in component:
            
            if last_species == species:
                AddSynonym( rep_transcript, transcript, map_synonyms, map_to_synonyms )                
            else:
                new_component.append( transcript )
                last_species = species
                rep_transcript = transcript
                
        new_components.append( new_component )
        
    components = new_components

    return components, map_synonyms

###########################################################
def updateGraphs( transcript1, transcript2,
                  weight,
                  map_transcript2id,                  
                  map_id2transcripts,
                  ids, graph,
                  reciprocal_best):

    id1 = map_transcript2id( transcript1 )
    id2 = map_transcript2id( transcript2 )    

    if id1 not in map_id2transcripts: map_id2transcripts[id1] = {}
    if id2 not in map_id2transcripts: map_id2transcripts[id2] = {}

    map_id2transcripts[id1][transcript1] = 1
    map_id2transcripts[id2][transcript2] = 1        

    ids[id1] = 1
    ids[id2] = 1

    key = "%s-%s" % (id1, id2)
    if key not in graph: graph[key] = []
    graph[key].append( (weight, transcript1, transcript2) )

    key = "%s-%s" % (id2, id1)
    if key not in graph: graph[key] = []
    graph[key].append( (weight, transcript1, transcript2) )

    ## compute reciprocal best matches
    if transcript1 not in reciprocal_best: reciprocal_best[transcript1] = {}
    if id2 not in reciprocal_best[transcript1]:
        reciprocal_best[transcript1][id2] = (weight, transcript2)
    else:
        if weight < reciprocal_best[transcript1][id2][0]:
            reciprocal_best[transcript1][id2] = (weight, transcript2)

    if transcript2 not in reciprocal_best: reciprocal_best[transcript2] = {}
    if id1 not in reciprocal_best[transcript2]:
        reciprocal_best[transcript2][id1] = (weight, transcript1)
    else:
        if weight < reciprocal_best[transcript2][id1][0]:
            reciprocal_best[transcript2][id1] = (weight, transcript1)

###########################################################
def getReciprocalBestComponents( reciprocal_best, map_transcript2id ):
                
    ## compute reciprocal best match graph
    subgraph = networkx.Graph()        
    for transcript, ids in reciprocal_best.items():
        
        subgraph.add_node( transcript )

        this_id = map_transcript2id( transcript )
        
        for other_id, best in ids.items():

            if this_id == other_id: continue

            best_weight, best_transcript = best
            if reciprocal_best[best_transcript][this_id][1] == transcript:
                subgraph.add_edge( transcript, best_transcript )
                
    ## compute components
    return networkx.connected_components( subgraph )

#############################################################################################
def getPrunedComponents( components, map_transcript2id,
                         map_priority,
                         map_synonyms, map_to_synonyms, options ):
    """eliminate sequences from the same species in each component.

    Take the highest quality in case of conflict.
    """
    new_components = []

    ## sort by transcript and quality
    for component in components:

        cc = []
        for transcript in component:
            s, t, g, q = transcript.split(options.separator)
            id = map_transcript2id( transcript )
            cc.append( (id, map_priority[q], transcript ) )
        
        cc.sort()
        new_component = []
        last_id = None
        rep_transcript = None
        for transcript in component:

            id = map_transcript2id(transcript)
            
            if last_id == id:
                AddSynonym( rep_transcript, transcript, map_synonyms, map_to_synonyms )                
            else:
                new_component.append( transcript )
                last_id = id
                rep_transcript = transcript
                
        new_components.append( new_component )
        
    return new_components
    
###########################################################
def clusterByReciprocity( links, map_transcript2id, options ):
    """given a graph of links between transcripts, split into
    clusters by transcripts."""

    ## second parse for actual processing. Synonyms are mapped

    map_id2transcripts = {}
    ids = {}
    graph = {}
    reciprocal_best = {}
    map_synonyms = {}
    map_to_synonyms = {}

    map_priority = {}
    for x in range( len(options.quality_priority) ):
        map_priority[options.quality_priority[x]] = x

    ninput, noutput, nskipped = 0, 0, 0
    for transcript1, transcript2, weight in links:

        ninput += 1
        weight = float(weight)

        if options.min_weight != None and weight < options.min_weight:
            nskipped += 1
            continue

        if options.max_weight != None and weight > options.max_weight:
            nskipped += 1
            continue

        if transcript1 > transcript2: transcript1, transcript2 = transcript2, transcript1
        noutput += 1
        updateGraphs( transcript1, transcript2, weight,
                      map_transcript2id,
                      map_id2transcripts, ids, graph, reciprocal_best )
        
    if options.loglevel >= 1:
        options.stdlog.write( "# ninput=%i, noutput=%i, nskipped=%i, nids=%i\n" % (ninput, noutput, nskipped, len(ids) ))

    components = getReciprocalBestComponents( reciprocal_best, map_transcript2id )

    if options.loglevel >= 1:
        options.stdlog.write( "# component sizes before pruning: %s\n" % ",".join(map( lambda x: str(len(x)), components)))

    components = getPrunedComponents( components, map_transcript2id,
                                      map_priority,
                                      map_synonyms, map_to_synonyms,
                                      options )
    
    if options.loglevel >= 1:
        options.stdlog.write( "# component sizes after pruning: %s\n" % ",".join(map( lambda x: str(len(x)), components)))

    return components, map_synonyms

##-------------------------------------------------------------------------------
if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: graph_cluster_by_species.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.add_option("-m", "--genome-master", dest="genome_master", type="string",
                      help="genome to use as master." )

    parser.add_option("-s", "--filename-synonyms", dest="filename_synonyms", type="string",
                      help="output filename for synonyms." )

    parser.add_option("-u", "--filename-summary", dest="filename_summary", type="string",
                      help="output filename of component summary." )

    parser.add_option("-w", "--synonym-weight", dest="synonym_weight", type="float",
                      help="weight for grouping together synonyms." )

    parser.add_option("-q", "--quality-priority", dest="quality_priority", type="string",
                      help="priority list of status codes for sorting.")

    parser.add_option("--min-weight", dest="min_weight", type="float",
                      help="minimum weight to use.")

    parser.add_option("--max-weight", dest="max_weight", type="float",
                      help="maximum weight to use.")

    parser.add_option("-o", "--mode", dest="mode", type="choice",
                      choices=("species", "genes"),
                      help="clustering mode - species or genes.")
    
    parser.set_defaults( \
        genome_master = None,
        synonym_weight = 0,
        filename_synonyms = None,
        filename_summary = None,
        quality_priority="CG,PG,SG,RG,CP,PP,SP,RP,CF,PF,SF,UG,UP,UF,BF,UK",
        mode="species",
        min_weight = None,
        max_weight = None,
        separator = "|")
        
    (options, args) = E.Start( parser )

    options.quality_priority = options.quality_priority.split(",")

    ###############################################################
    ###############################################################
    ###############################################################        
    ## input
    ###############################################################                

    links = map(lambda x: x[:-1].split("\t"), filter( lambda x: x[0] != "#", sys.stdin.readlines()))

    ###############################################################
    ###############################################################
    ###############################################################        
    ## clustering
    ###############################################################                

    if options.mode == "species":

        def map_transcript2id( transcript, separator= "|" ):
            s, t, g, q = transcript.split(separator)
            return s
        
        components, map_synonyms = clusterBySpecies( links, options )

    elif options.mode == "genes":
        
        def map_transcript2id( transcript, separator= "|" ):
            s, t, g, q = transcript.split(separator)
            return "%s%s%s" % (s, separator, g )
        
        components, map_synonyms = clusterByReciprocity( links, map_transcript2id, options )

    ###############################################################
    ###############################################################
    ###############################################################        
    ## output
    ###############################################################                
    n = 1
    options.stdout.write( "id\tcomponent\n" )    
    for component in components:
        for c in component:
            options.stdout.write( "%s\t%i\n" % (c, n))
        n += 1

    n = 1
    if options.filename_summary:
        outfile = open(options.filename_summary, "w")
        outfile.write("component\tsize\tnspecies\tnmaster\tnsynonyms\n" )
        for component in components:
            species = map( lambda x: x.split(options.separator)[0], component)
            nsynonyms = 0
            for c in component:
                if c in map_synonyms: nsynonyms += len(map_synonyms[c])
            outfile.write( "%i\t%i\t%i\t%i\t%i\n" % ( n,
                                                      len(component),
                                                      len(species),
                                                      len(filter( lambda x: x == options.genome_master, species)),
                                                      nsynonyms))

            n += 1
        
    if options.filename_synonyms and len(map_synonyms):
        outfile = open(options.filename_synonyms, "w")
        outfile.write("rep\tmem\n")
        for k,v in map_synonyms.items():
            for m in v:
                outfile.write( "%s\t%s\n" % (k, m))
        outfile.close()
        
    if options.loglevel >= 1:
        
        options.stdlog.write( "# ncomponents=%i, nsynonyms=%i\n" % (len(components), len(map_synonyms)))
        options.stdlog.write( "# final component sizes: %s\n" % ",".join(map( lambda x: str(len(x)), components)))

    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
bam2fastq.py - 
=============================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

Convert a BAM file to a FASTQ files.

Usage
-----

Example::

   python bam2fastq.py in.bam out.1.fastq out.2.fastq

This command converts the BAM file in.bam into fastq files containing forward reads (out.1.fastq) and reverse reads 
(out.2.fastq). 

Type::

   python cgat_script_template.py --help

for command line help.

Documentation
-------------

This tool converts a BAM file containing paired-end reads into forward and reverse fastq files.

For example::

   python bam2fastq.py in.bam out.1.fastq out.2.fastq 

Code
----

'''

import os
import sys
import re
import optparse
import gzip

import CGAT.Experiment as E
import CGAT.IOTools as IOTools

import pysam

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option("-i", "--test-option", dest="test_option", type="string",
                      help="test option [default=%default]."  )

    parser.set_defaults(
        test_option = "test"
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    ## do sth
    assert len(args) == 3, "expected three command line arguments" 

    samfile = pysam.Samfile( args[0], "rb" )
    outstream1 = IOTools.openFile( args[1], "w" )
    outstream2 = IOTools.openFile( args[2], "w" )

    c = E.Counter()
    for read in samfile.fetch():
        c.input += 1
        if read.mate_is_unmapped:
            c.is_unmapped += 1
            if read.is_read1:
                seq1, seq2 = read.seq, "N" * read.qlen 
                qual1, qual2 = read.qual, "B" * read.qlen
            else:
                seq1, seq2 = "N" * read.qlen, read.seq
                qual1, qual2 = "B" * read.qlen, read.qual
        else:
            if read.is_read2: continue
            try:
                mate = samfile.mate( read )
                c.found += 1
            except ValueError, msg:
                mate = None
                c.failed += 1
                
            if mate:
                seq1, seq2 = read.seq, mate.seq
                qual1, qual2 = read.qual, mate.qual
            else:
                seq1, seq2 = read.seq, "N" * read.qlen 
                qual1, qual2 = read.qual, "B" * read.qlen

        c.output += 1
        outstream1.write( "@%s\n%s\n+\n%s\n" % (read.qname, seq1, qual1 ) )
        outstream2.write( "@%s\n%s\n+\n%s\n" % (read.qname, seq2, qual2 ) )
        
    E.info( "%s" % str(c) )

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )
